from livesettings import *
from django.utils.translation import ugettext_lazy as _

# this is so that the translation utility will pick up the string
gettext = lambda s: s
_strings = (gettext('CreditCard'), gettext('Credit Card'))

PAYMENT_GROUP = ConfigurationGroup('PAYMENT_AUTHORIZENET', 
    _('Authorize.net Payment Settings'), 
    ordering=101)

config_register_list(

    StringValue(PAYMENT_GROUP, 
        'CONNECTION',
        description=_("Submit to URL"),
        help_text=_("""This is the address to submit live transactions."""),
        default='https://secure.authorize.net/gateway/transact.dll'),

    StringValue(PAYMENT_GROUP, 
        'CONNECTION_TEST',
        description=_("Submit to Test URL"),
        help_text=("""A Quick note on the urls.<br/>
If you are posting to https://test.authorize.net/gateway/transact.dll,
and you are not using an account whose API login ID starts with
&quot;cpdev&quot; or &quot;cnpdev&quot;, you will get an Error 13 message. 
Make sure you are posting to https://certification.authorize.net/gateway/transact.dll
for test transactions if you do not have a cpdev or cnpdev.
"""),
        default='https://test.authorize.net/gateway/transact.dll'),


    BooleanValue(PAYMENT_GROUP, 
        'SSL', 
        description=_("Use SSL for the checkout pages?"), 
        default=False),
    
    BooleanValue(PAYMENT_GROUP, 
        'LIVE', 
        description=_("Accept real payments"),
        help_text=_("False if you want to submit to the test urls.  NOTE: If you are testing, then you can use the cc# 4222222222222 to force a bad credit card response.  If you use that number and a ccv of 222, that will force a bad ccv response from authorize.net"),
        default=False),

    BooleanValue(PAYMENT_GROUP, 
        'SIMULATE', 
        description=_("Force a test post?"),
        help_text=_("True if you want to submit to the live url using a test flag, which won't be accepted."),
        default=False),

    ModuleValue(PAYMENT_GROUP,
        'MODULE',
        description=_('Implementation module'),
        hidden=True,
        default = 'payment.modules.authorizenet'),
    
    StringValue(PAYMENT_GROUP,
        'KEY',
        description=_("Module key"),
        hidden=True,
        default = 'AUTHORIZENET'),

    StringValue(PAYMENT_GROUP,
        'LABEL',
        description=_('English name for this group on the checkout screens'),
        default = 'Credit Cards',
        help_text = _('This will be passed to the translation utility')),

    StringValue(PAYMENT_GROUP,
        'URL_BASE',
        description=_('The url base used for constructing urlpatterns which will use this module'),
        default = r'^credit/'),

    MultipleStringValue(PAYMENT_GROUP,
        'CREDITCHOICES',
        description=_('Available credit cards'),
        choices = (
            (('American Express', 'American Express')),
            (('Visa','Visa')),
            (('Mastercard','Mastercard')),
            (('Discover','Discover'))),
        default = ('Visa', 'Mastercard', 'Discover')),
    
    StringValue(PAYMENT_GROUP, 
        'LOGIN', 
        description=_('Your authorize.net transaction login'),
        default=""),
    
    StringValue(PAYMENT_GROUP, 
        'TRANKEY', 
        description=_('Your authorize.net transaction key'),
        default=""),
        
    BooleanValue(PAYMENT_GROUP,
        'CAPTURE',
        description=_('Capture Payment immediately?'),
        default=True,
        help_text=_('IMPORTANT: If false, a capture attempt will be made when the order is marked as shipped."')),

    BooleanValue(PAYMENT_GROUP,
        'EXTRA_LOGGING',
        description=_("Verbose logs"),
        help_text=_("Add extensive logs during post."),
        default=False)
)

ARB_ENABLED = config_register(    
    BooleanValue(PAYMENT_GROUP,
        'ARB',
        description=_('Enable ARB?'),
        default=False,
        help_text=_('Enable ARB processing for setting up subscriptions.  You must have this enabled in your Authorize account for it to work.')))

config_register(
    StringValue(PAYMENT_GROUP, 
        'ARB_CONNECTION',
        description=_("Submit to URL (ARB)"),
        help_text=_("""This is the address to submit live transactions for ARB."""),
        requires=ARB_ENABLED,
        default='https://api.authorize.net/xml/v1/request.api'))

config_register(    
    StringValue(PAYMENT_GROUP, 
        'ARB_CONNECTION_TEST',
        description=_("Submit to Test URL (ARB)"),
        help_text=_("""This is the address to submit test transactions for ARB."""),
        requires=ARB_ENABLED,
        default='https://apitest.authorize.net/xml/v1/request.api'))




# Copyright (C) 2004,2005 by SICEm S.L.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import operator
import xml.dom.minidom

import gtk
import gobject
from kiwi.ui.dialogs import BaseDialog

from gazpacho.i18n import _
from gazpacho.propertyeditor import PropertyEditorDialog
from gazpacho.util import select_iter
from gazpacho.command import Command
from gazpacho.commandmanager import command_manager

# XXX: Remove
def xml_filter_nodes(nodes, node_type):
    return [node for node in nodes if node.nodeType == node_type]

class UIEditor(PropertyEditorDialog):
    def __init__(self):
        # we can have a menubar or a toolbar here
        self.is_menu = True

        PropertyEditorDialog.__init__(self)
        self.uim = None

    def set_gadget(self, gadget, proxy):
        super(UIEditor, self).set_gadget(gadget, proxy)

        self.uim = gadget.project.uim

        if isinstance(gadget.widget, gtk.MenuBar):
            self.set_title(_('Editing menubar %s') % gadget.name)
            self.is_menu = True
        elif isinstance(gadget.widget, gtk.Toolbar):
            self.set_title(_('Editing toolbar %s') % gadget.name)
            self.is_menu = False
        else:
            raise ValueError(_('UIEditor is only for toolbars and menubars'))

        self._load_ui()

    # internal methods
    def _create_widgets(self):
        "Build the dialog window interface widgets"
        #XXX: Could this method be called build_editor_dialog?
        hbox = gtk.HBox(spacing=6)

        sw = gtk.ScrolledWindow()
        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        hbox.pack_start(sw, True, True)

        self.model = gtk.TreeStore(object)
        self.treeview = gtk.TreeView(self.model)
        self.treeview.connect('button-press-event',
                              self._on_treeview__button_press_event)
        self.treeview.connect('key-press-event',
                              self._on_treeview__key_press_event)
        selection = self.treeview.get_selection()
        selection.connect('changed', self._on_selection_changed)
        self.treeview.set_size_request(300, 300)
        sw.add(self.treeview)
        self.treeview.set_headers_visible(False)
        renderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn()
        self.treeview.append_column(column)
        column.pack_start(renderer)
        column.set_cell_data_func(renderer, self._cell_data_function)

        v_button_box = gtk.VButtonBox()
        v_button_box.set_layout(gtk.BUTTONBOX_START)
        self.up = gtk.Button(stock=gtk.STOCK_GO_UP)
        self.up.connect('clicked', self._on_up_clicked)
        v_button_box.pack_start(self.up)
        self.down = gtk.Button(stock=gtk.STOCK_GO_DOWN)
        self.down.connect('clicked', self._on_down_clicked)
        v_button_box.pack_start(self.down)
        self.left = gtk.Button(stock=gtk.STOCK_GO_BACK)
        self.left.connect('clicked', self._on_left_clicked)
        self.right = gtk.Button(stock=gtk.STOCK_GO_FORWARD)
        self.right.connect('clicked', self._on_right_clicked)
        if self.is_menu:
            v_button_box.pack_start(self.left)
            v_button_box.pack_start(self.right)

#        hbox.pack_start(v_button_box, False, False)

        self.vbox.pack_start(hbox, True, True)

        h_button_box = gtk.HButtonBox()
        h_button_box.set_spacing(6)
        h_button_box.set_layout(gtk.BUTTONBOX_START)
        self.add = gtk.Button(stock=gtk.STOCK_ADD)
        self.add.connect('clicked', self._on_add_clicked)
        h_button_box.pack_start(self.add)
        self.remove = gtk.Button(stock=gtk.STOCK_REMOVE)
        self.remove.connect('clicked', self._on_remove_clicked)
        h_button_box.pack_start(self.remove)
        self.vbox.pack_start(h_button_box, False, False)

        label = gtk.Label()
        label.set_markup('<small>%s</small>' % \
                         _('Press Escape to unselected an item'))
        self.vbox.pack_start(label, False, False)

    def _on_treeview__button_press_event(self, treeview, event):
        result = treeview.get_path_at_pos(int(event.x), int(event.y))
        if not result:
            selection = treeview.get_selection()
            selection.unselect_all()

    def _on_treeview__key_press_event(self, treeview, event):
        if event.keyval == gtk.keysyms.Escape:
            selection = treeview.get_selection()
            selection.unselect_all()

    def _load_node(self, xml_node, parent_iter):
        new_iter = self.model.append(parent_iter, (xml_node,))
        for child_node in xml_filter_nodes(xml_node.childNodes,
                                           xml_node.ELEMENT_NODE):
            self._load_node(child_node, new_iter)

    def _load_ui(self):
        # XXX right now we only allow to edit the initial state
        ui = self.uim.get_ui(self.gadget, 'initial-state')

        self.model.clear()
        if ui:
            xml_string = ui[0]
            self.xml_doc = xml.dom.minidom.parseString(xml_string)
            self.xml_doc.normalize()
            self.root_node = self.xml_doc.documentElement
            typename = self.root_node.tagName
        else:
            # create empty XML document
            self.xml_doc = xml.dom.getDOMImplementation().createDocument(None,
                                                                         None,
                                                                         None)
            if self.is_menu:
                typename = 'menubar'
            else:
                typename = 'toolbar'
            self.root_node = self._create_xml_node(self.xml_doc, typename,
                                                   self.gadget.name)
        for xml_node in xml_filter_nodes(self.root_node.childNodes,
                                         self.root_node.ELEMENT_NODE):
            self._load_node(xml_node, None)

        self._set_buttons_sensitiviness(typename)

    def _cell_data_function(self, column, cell, model, current_iter):
        xml_node = model[current_iter][0]
        typename = xml_node.tagName
        name = xml_node.getAttribute('name')
        if typename == 'separator':
            name = '<separator>'
        cell.set_property('text', name)

    def _get_selected_iter(self):
        selection = self.treeview.get_selection()
        return selection.get_selected()[1]

    def _create_xml_node(self, parent, typename, name):
        element = self.xml_doc.createElement(typename)
        element.setAttribute('name', name)
        if typename != 'separator':
            element.setAttribute('action', name)
        return parent.appendChild(element)

    def _set_buttons_sensitiviness(self, typename):
        "Set edit buttons sensitiveness depending on class"

        buttons = (self.add, self.remove, self.up, self.down, self.left,
                   self.right)
        sensitivity = {
            'menubar': (True, False, False, False, False, False),
            'menu': (True, True, True, True, False, True),
            'menuitem': (False, True, True, True, True, False),
            'toolbar':(True, False, False, False, False, False),
            'toolitem': (False, True, True, True, False, False ),
            'separator': (False, True, True, True, False, False),
            }
        for i, s in enumerate(sensitivity[typename]):
            buttons[i].set_sensitive(s)

    def _set_ui(self):
        ui_string = self.root_node.toxml()
        cmd = CommandUpdateUIDefinitions(self.gadget, ui_string,
                                         'initial-state')
        command_manager.execute(cmd, self.gadget.project)

    # callbacks
    def _on_selection_changed(self, selection):
        "Get selected element and set edit buttons sensitiveness for it"

        model, selected_iter = selection.get_selected()
        if selected_iter is not None:
            xml_node = model[selected_iter][0]
            name = xml_node.tagName
        else:
            name = self.root_node.tagName

        self._set_buttons_sensitiviness(name)

    def _on_add_clicked(self, button):
        selected_iter = self._get_selected_iter()
        # we are adding a toplevel
        selected_node = self.root_node
        if self.is_menu:
            if selected_iter:
                dialog_class = AddMenuitemDialog
                typename = 'menuitem'
                selected_node = self.model[selected_iter][0]
            else:
                typename = 'menu'
                dialog_class = AddMenuDialog
        else:
            dialog_class = AddToolitemDialog
            typename = 'toolitem'
            if selected_iter:
                selected_node = self.root_node
                selected_iter = None

        dialog = dialog_class(self.uim.get_action_groups(), self)
        response = dialog.run()
        if response != gtk.RESPONSE_OK:
            dialog.destroy()
            return

        if dialog.use_separator():
            typename = 'separator'
            name = 'sep'
        else:
            name = dialog.get_name()
        node = self._create_xml_node(selected_node, typename, name)
        new_iter = self.model.append(selected_iter, (node,))
        select_iter(self.treeview, new_iter)
        self._set_ui()

        dialog.destroy()

    def _on_remove_clicked(self, button):
        selected_iter = self._get_selected_iter()
        if selected_iter is None:
            return

        xml_node = self.model[selected_iter][0]
        self.model.remove(selected_iter)

        parent_node = xml_node.parentNode
        parent_node.removeChild(xml_node)

        self._set_ui()

    def _on_up_clicked(self, button):
        selected_iter = self._get_selected_iter()
        if selected_iter is None:
            return

        xml_node = self.model[selected_iter][0]
        parent_node = xml_node.parentNode
        previous_sibling = xml_node.previousSibling
        if previous_sibling is not None:
            print(previous_sibling)
            tmp_node = parent_node.removeChild(xml_node)
            parent_node.insertBefore(tmp_node, previous_sibling)
            # not update the TreeView
            selected_path = self.model.get_path(selected_iter)
            print(selected_path, type(selected_path))
            sibling_iter = self.model[selected_path[0] - 1].iter
            self.model.move_before(selected_iter, sibling_iter)

            self._set_ui()

    def _on_down_clicked(self, button):
        print('not implemented yet')

    def _on_left_clicked(self, button):
        print('not implemented yet')

    def _on_right_clicked(self, button):
        print('not implemented yet')


class ChooseActionDialog(BaseDialog):
    """Dialog to choose an action from a list"""
    def __init__(self, action_groups=[], toplevel=None):
        BaseDialog.__init__(self, parent=toplevel,
                            buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                     gtk.STOCK_OK, gtk.RESPONSE_OK))
        self.set_resizable(False)

        self.label = gtk.Label(_('Choose an action')+':')
        self.vbox.pack_start(self.label, False)

        # create a combo with all the actions
        model = gtk.ListStore(str)
        self.actions = gtk.ComboBox(model)
        renderer = gtk.CellRendererText()
        self.actions.pack_start(renderer)
        self.actions.add_attribute(renderer, 'text', 0)

        for action_group in action_groups:
            actions = action_group.get_actions()
            actions.sort(key=operator.attrgetter('name'))
            for action in actions:
                model.append(('%s/%s' % (action_group.name, action.name),))

        self.actions.set_active(0)

        self.vbox.pack_start(self.actions)

        self.set_default_response(gtk.RESPONSE_OK)

    def get_selected_action_name(self):
        model = self.actions.get_model()
        active_iter = self.actions.get_active_iter()
        if active_iter is not None:
            name = model[active_iter][0]
            action_name = name.split('/')[1]
            return action_name

    def use_separator(self):
        """Subclasses will probably override this"""
        return False

gobject.type_register(ChooseActionDialog)

class AddMenuDialog(ChooseActionDialog):
    def __init__(self, action_groups=[], toplevel=None):
        ChooseActionDialog.__init__(self, action_groups, toplevel)

        self.set_title(_('Adding a menu'))
        self.vbox.show_all()

    def get_name(self):
        return self.get_selected_action_name()

gobject.type_register(AddMenuDialog)

class AddToolitemDialog(AddMenuDialog):
    def __init__(self, action_groups=[], toplevel=None):
        AddMenuDialog.__init__(self, action_groups, toplevel)

        # put the combo in other place
        self.vbox.remove(self.actions)
        self.vbox.remove(self.label)

        hbox = gtk.HBox(spacing=6)
        self.select_action = gtk.RadioButton(None, _('Action'))
        self.select_separator = gtk.RadioButton(self.select_action,
                                                _('Separator'))

        hbox.pack_start(self.select_action, False, False)
        hbox.pack_start(self.actions)

        self.vbox.pack_start(hbox, False, False)
        self.vbox.pack_start(self.select_separator, False, False)

        self.set_title(_('Adding a toolitem'))
        self.vbox.show_all()

        if len(self.actions.get_model()) > 0:
            self.select_action.set_sensitive(True)
            self.select_action.set_active(True)
        else:
            self.select_action.set_sensitive(False)
            self.select_action.set_active(False)
            self.select_separator.set_active(True)

    def use_separator(self):
        return self.select_separator.get_active()

gobject.type_register(AddToolitemDialog)

class AddMenuitemDialog(AddToolitemDialog):
    def __init__(self, action_groups=[], toplevel=None):
        AddToolitemDialog.__init__(self, action_groups, toplevel)

        self.set_title(_('Adding a menuitem'))
        self.vbox.show_all()

gobject.type_register(AddMenuitemDialog)


class CommandUpdateUIDefinitions(Command):
    """
    Update the UI definitions for a UIManager based widget.
    """

    def __init__(self, gadget, ui_defs, ui_name, refresh=True):
        Command.__init__(self, _("Edit UI definitions"))
        self._gadget = gadget
        self._defs = ui_defs
        self._ui_name = ui_name
        # If this is set to False the UIM won't refresh the gtk widget and
        # you are responsible of doing so at some point
        self._refresh = refresh

    def execute(self):
        uim = self._gadget.project.uim
        old_defs = uim.get_ui(self._gadget, self._ui_name)[0]
        uim.update_ui(self._gadget, self._defs, self._ui_name, self._refresh)
        self._defs = old_defs

    def unifies(self, other):
        if isinstance(other, CommandUpdateUIDefinitions):
            return (self._gadget is other._gadget
                    and self._ui_name == other._ui_name)
        return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Require MoinMoin 1.8.9
from MoinMoin.request import request_wsgi
from MoinMoin.server.server_wsgi import moinmoinApp
#from MoinMoin.server.server_wsgi import WsgiConfig as ConfigBase
from notmm.controllers.base import BaseController
#from notmm.utils.wsgilib import HTTPResponse
import logging
log = logging.getLogger(__name__)

__all__ = ['MoinMoinController']

#debug_moinmoin_version = '1.8.9'

class MoinMoinController(BaseController):
    """
    A BaseController extension serving a MoinMoin wiki app.
    """

    #debug = False
    logger = log

    def application(self, environ, start_response):
        """Creates a mini MoinMoin wiki handler with generic file 
        upload support.

        """
        return moinmoinApp(environ, start_response)



# Suck in default django settings
from django.conf.global_settings import *

from pkg_resources import resource_filename

import os

DEBUG = True
USE_I18N = True

# Custom error handlers mapping
CUSTOM_ERROR_HANDLERS = (
    #('handle401', 'sandbox.handlers.handle401'),
    #('handle403', 'sandbox.handlers.handle403'),
    ('handle404', 'sandbox.handlers.handle404'),
    ('handle500', 'sandbox.handlers.handle500')
    )

# List of processors used by RequestContext to populate the context.
# Each one should be a callable that takes the request object as its
# only parameter and returns a dictionary to add to the context.
#TEMPLATE_CONTEXT_PROCESSORS = (
#    'django.contrib.auth.context_processors.auth',
#    'blogengine.contrib.i18n.context_processors.i18n'
#    )
TEMPLATE_CONTEXT_PROCESSORS = ()

# For backward-compatibility with Django
TEMPLATE_DIRS = (
    resource_filename('sandbox', 'templates'),
)

ROOT_URLCONF = 'sandbox.config.urls'

MIDDLEWARE_CLASSES = ()

#SITE_ID = 1

#a new method for changing django INSTALLED_APPS behavior 
#is described here:
#   [type] the model type (default: "native") (Django legacy orm)
#   [debug] turn debug on for this app..
#   [profile] turn profiling on for the app
#INSTALLED_APPS = (
#    'django.contrib.auth', 
#    'django.contrib.admin',
#    'django.contrib.sessions',
#    'django.contrib.contenttypes'
#    )
INSTALLED_APPS = ()

ENABLE_TIDYLIB = False

ENABLE_LOGGING = True
LOGGING_CALLBACK = 'rootLogger'



from django import forms
from livesettings import *
import logging

log = logging.getLogger('configuration')

class SettingsEditor(forms.Form):
    "Base editor, from which customized forms are created"

    def __init__(self, *args, **kwargs):
        settings = kwargs.pop('settings') 
        super(SettingsEditor, self).__init__(*args, **kwargs)
        flattened = []
        groups = []
        for setting in settings:
            if isinstance(setting, ConfigurationGroup):
                for s in setting:
                    flattened.append(s)
            else:
                flattened.append(setting)
                    
        for setting in flattened:
            # Add the field to the customized field list
            kw = {
                'label': setting.description,
                'help_text': setting.help_text,
                # Provide current setting values for initializing the form
                'initial': setting.editor_value
            }
            field = setting.make_field(**kw)
            
            k = '%s__%s' % (setting.group.key, setting.key)
            self.fields[k] = field
            if not setting.group in groups:
                groups.append(setting.group)
            #log.debug("Added field: %s = %s" % (k, str(field)))

        self.groups = groups


# Copyright (C) 2007-2009 Etienne Robillard <robillard.etienne@gmail.com>
# All rights reserved.
#
# Please see the "LICENSE" and "NOTICE" files included in the source 
# distribution for details on licensing info.
"""Schema for the BlogEngine app.

TODO:
* BlogEntry -- Can be published/hidden.
* BlogEntryImage -- A image that belongs to a attached BlogEntry.

General stuff:
* Author -- The author of the BlogEntry.
* Comment -- A comment. 
* Tag -- A tag.

"""

from schevo.schema import *
schevo.schema.prep(locals())

class Author(E.Entity):
    
    name = f.string()
    # TODO: add a emailstring() method to support e-mail bytestrings in utf8
    #email = f.string()

    _key(name)

    #def x_blogentries(self):
    #    return [casting.movie
    #            for casting in self.m.movie_castings()]

class Tag(E.Entity):

    name = f.string()
    description = f.string(required=False)

    _key(name)
    
class BlogEntry(E.Entity):

    title = f.string()
    pub_date = f.date()
    author = f.entity('Author')
    
    short_description = f.string(required=False)
    body = f.string(multiline=True)

    _key(title)

    #def x_actors(self):
    #    return [casting.actor
    #            for casting in self.m.movie_castings()]

    def __unicode__(self):
        return u'%s (%i)' % (self.title, self.pub_date.year)

E.BlogEntry._sample = [
    ('This is a test using notmm-schevo', '2009-05-04', 
      ('Etienne Robillard', ), 'Just a test', 'Long description here'),
     ('2nd post!', '2009-05-04',
      ('Etienne Robillard', ), '2nd test with notmm-schevo', 'so far so good :)')]




from decimal import Decimal
from django.test import TestCase
from keyedcache import cache_delete
from livesettings import config_get
from models import *
from product.models import Product
from satchmo_store.contact.models import AddressBook, Contact
from satchmo_store.shop.models import Order, OrderItem, OrderPayment
from satchmo_store.shop.tests import make_test_order, make_order_payment
import datetime
import logging
log = logging.getLogger('tax.test')

def _set_percent_taxer(percent):
    
    cache_delete()
    tax = config_get('TAX','MODULE')
    tax.update('tax.modules.percent')
    pcnt = config_get('TAX', 'PERCENT')
    pcnt.update(percent)
    shp = config_get('TAX', 'TAX_SHIPPING')
    shp.update(False)
    
    return shp

class TaxTest(TestCase):
    
    fixtures = ['l10n_data.xml', 'test_tax.yaml', 'test_discount.yaml']
            
    def tearDown(self):
        cache_delete()
            
    def testAreaCountries(self):
        """Test Area tax module"""
        cache_delete()
        tax = config_get('TAX','MODULE')
        tax.update('tax.modules.area')
        
        order = make_test_order('DE', '', include_non_taxed=True)
        
        order.recalculate_total(save=False)
        price = order.total
        subtotal = order.sub_total
        tax = order.tax
        
        self.assertEqual(subtotal, Decimal('105.00'))
        self.assertEqual(tax, Decimal('20.00'))
        # 100 + 10 shipping + 20 tax
        self.assertEqual(price, Decimal('135.00'))
        
        taxes = order.taxes.all()
        self.assertEqual(2, len(taxes))
        t1 = taxes[0]
        t2 = taxes[1]
        self.assert_('Shipping' in (t1.description, t2.description))
        if t1.description == 'Shipping':
            tship = t1
            tmain = t2
        else:
            tship = t2
            tmain = t1
        self.assertEqual(tmain.tax, Decimal('20.00'))
        self.assertEqual(tship.tax, Decimal('0.00'))
        
        order = make_test_order('CH', '')
        
        order.recalculate_total(save=False)
        price = order.total
        subtotal = order.sub_total
        tax = order.tax
        
        self.assertEqual(subtotal, Decimal('100.00'))
        self.assertEqual(tax, Decimal('16.00'))
        # 100 + 10 shipping + 16 tax
        self.assertEqual(price, Decimal('126.00'))

        taxes = order.taxes.all()
        self.assertEqual(2, len(taxes))
        t1 = taxes[0]
        t2 = taxes[1]
        self.assert_('Shipping' in (t1.description, t2.description))
        if t1.description == 'Shipping':
            tship = t1
            tmain = t2
        else:
            tship = t2
            tmain = t1
        
        self.assertEqual(tmain.tax, Decimal('16.00'))
        self.assertEqual(tship.tax, Decimal('0.00'))
        
    def testDuplicateAdminAreas(self):
        """Test the situation where we have multiple adminareas with the same name"""
        cache_delete()
        tax = config_get('TAX','MODULE')
        tax.update('tax.modules.area')

        order = make_test_order('GB', 'Manchester')

        order.recalculate_total(save=False)
        price = order.total
        subtotal = order.sub_total
        tax = order.tax

        self.assertEqual(subtotal, Decimal('100.00'))
        self.assertEqual(tax, Decimal('20.00'))
        # 100 + 10 shipping + 20 tax
        self.assertEqual(price, Decimal('130.00'))

        taxes = order.taxes.all()
        self.assertEqual(2, len(taxes))
        t1 = taxes[0]
        t2 = taxes[1]
        self.assert_('Shipping' in (t1.description, t2.description))
        if t1.description == 'Shipping':
            tship = t1
            tmain = t2
        else:
            tship = t2
            tmain = t1
        self.assertEqual(tmain.tax, Decimal('20.00'))
        self.assertEqual(tship.tax, Decimal('0.00'))

    def testPercent(self):
        """Test percent tax without shipping"""
        shp = _set_percent_taxer('10')
        order = make_test_order('US', 'TX')
        
        order.recalculate_total(save=False)
        price = order.total
        subtotal = order.sub_total
        tax = order.tax        
        
        self.assertEqual(subtotal, Decimal('100.00'))
        self.assertEqual(tax, Decimal('10.00'))
        # 100 + 10 shipping + 10 tax
        self.assertEqual(price, Decimal('120.00'))
        
        taxes = order.taxes.all()
        self.assertEqual(1, len(taxes))
        self.assertEqual(taxes[0].tax, Decimal('10.00'))
        self.assertEqual(taxes[0].description, r'10%')
        
        
    def testPercentShipping(self):
        """Test percent tax with shipping"""
        shp = _set_percent_taxer('10')
        shp.update(False)

        order = make_test_order('US', 'TX')
        shp.update(True)
        order.recalculate_total(save=False)
        price = order.total
        tax = order.tax
        
        self.assertEqual(tax, Decimal('11.00'))
        # 100 + 10 shipping + 11 tax
        self.assertEqual(price, Decimal('121.00'))
        
        taxes = order.taxes.all()
        self.assertEqual(2, len(taxes))
        t1 = taxes[0]
        t2 = taxes[1]
        self.assert_('Shipping' in (t1.description, t2.description))
        if t1.description == 'Shipping':
            tship = t1
            tmain = t2
        else:
            tship = t2
            tmain = t1
            
        self.assertEqual(tmain.tax, Decimal('10.00'))
        self.assertEqual(tmain.description, r'10%')
        self.assertEqual(tship.tax, Decimal('1.00'))
        self.assertEqual(tship.description, 'Shipping')
        
    def testFractionalPercentShipping(self):
        """Test proper handling of taxes when using a fractional percent tax.  This can cause
        situations where the total is xx.xxxxx, the payment processor charges xx.xx, 
        leaving 00.00xxxx remaining."""
        
        log.debug('fractionalpercent')        
        shp = _set_percent_taxer('12.254')
        log.debug('making order')
        order = make_test_order('US', 'TX')
        order.recalculate_total(save=False)
        price = order.total
        tax = order.tax
        
        self.assertEqual(tax, Decimal('12.254'))
        self.assertEqual(order.balance, Decimal('122.26'))
        
        make_order_payment(order, amount=Decimal('122.26'))
        
        self.assertEqual(order.balance, Decimal("0.00"))
        self.assert_(order.paid_in_full)

        # try with a known bad case - from email
        shp = _set_percent_taxer('12.25')
        order = make_test_order('US', 'TX', price=Decimal('209.90'), quantity=1)
        order.recalculate_total(save=False)
        price = order.total
        tax = order.tax
        
        self.assertEqual(tax, Decimal('25.71275'))
        self.assertEqual(order.total, Decimal('245.61275'))
        self.assertEqual(order.balance, Decimal('245.62'))
        
        make_order_payment(order, amount=Decimal('245.62'))
        
        #self.assertEqual(order.balance, Decimal("0.00"))
        self.assert_(order.paid_in_full)
        


from django import forms
from django.core import urlresolvers
from django.utils.translation import ugettext, ugettext_lazy as _
from payment.utils import capture_authorizations
from livesettings import config_value
import logging

log = logging.getLogger('payment.listeners')

def form_terms_listener(sender, form=None, **kwargs):
    """Adds a 'do you accept the terms and conditions' checkbox to the form"""

    try:
        url = urlresolvers.reverse('shop_terms')
    except urlresolvers.NoReverseMatch:
        log.warn('To use the form_terms_listener, you must have a "shop_terms" url in your site urls')
        url = "#"
        
    link = u'<a target="_blank" href="%s"></a>' % ugettext('terms and conditions')
    form.fields['terms'] = forms.BooleanField(
        label=_('Do you accept the %(terms_link)s?') % {'terms_link' : link}, 
        widget=forms.CheckboxInput(), required=True)

def shipping_hide_if_one(sender, form=None, **kwargs):
    """Makes the widget for shipping hidden if there is only one choice."""

    log.warn("shipping_hide_if_one listener is deprecated, please configure this in your site settings in the shipping section.")
    
def capture_on_ship_listener(sender, oldstatus="", newstatus="", order=None, **kwargs):
    """Listen for a transition to 'shipped', and capture authorizations."""

    log.debug('heard satchmo_order_status_changed, %s=%s', oldstatus, newstatus)
    if oldstatus != 'Shipped' and newstatus == 'Shipped':
        capture_authorizations(order)



from django.core.management.base import NoArgsCommand
import os
import shutil

class Command(NoArgsCommand):
    help = "Copy the satchmo urls file to the current project."

    def handle_noargs(self, **options):
        import satchmo_store
        url_src = os.path.join(satchmo_store.__path__[0],'urls.py')
        url_dest = os.path.join(os.getcwd(), 'satchmo-urls.py')
        shutil.copyfile(url_src, url_dest)
        print "Copied %s to %s" % (url_src, url_dest)



import os

def normalize_path(path):
    """
    Converts the supplied path to lower case, removes any unnecessary slashes 
    and makes all slashes forward slashes.
    This is particularly useful for Windows systems.
    It is recommended to use in your settings if you are on Windows.
    
    MEDIA_ROOT = normalize_path(os.path.join(DIRNAME, 'static/'))
    """
    return os.path.normcase(os.path.normpath(path)).replace('\\', '/')



import time
import hashlib
import logging

#from pubsub import pub
#import beaker.session
#from notmm.utils.configparse import loadconf
from notmm.utils.wsgilib     import HTTPRedirectResponse
#from notmm.controllers.auth  import AuthCookieController

from notmm.utils.template    import direct_to_template
from authkit.authenticate   import valid_password
from authkit.permissions    import RemoteUser
from authkit.authorize      import NotAuthenticatedError

log = logging.getLogger(__name__)
#auth_conf = loadconf('development.ini', section='authkit')

__all__ = ['authenticate_user', 'logout', 'login', 'unauthorized']

def authenticate_user(request, username, password, tokens='', user_data=time.ctime,
    authfunc='paste.auth_tkt.set_user'):
    """Authenticate the user into the site and update the last_modified
    timestamp if authentication and authorization granted user access."""

    # Add the cookie middleware
    # secret = hashlib.md5(username).hexdigest()
    # request = AuthTKTMiddleware(request, secret)
    try:
        user_setter_func = request.environ[authfunc]
        if valid_password(request.environ, username, password):
            user_setter_func(username, tokens=tokens, user_data=user_data())
            #trigger function here to update the last_modified timestamp 
            log.debug('User %s has been authenticated and authorized access!!' % username)
        raise NotAuthenticatedError
    except (KeyError, Exception):
        raise NotAuthenticatedError
    return None
    
def logout(request, template_name='auth/logout.html',
    session_key='wsgioauth2.session', urlto='/'):
    if 'REMOTE_USER' in request.environ.keys():
        del request.environ['REMOTE_USER']
        log.debug('Session logout: deleted REMOTE_USER from session!')
    if 'HTTP_COOKIE' in request.environ.keys():
        del request.environ['HTTP_COOKIE']
        log.debug('deleted HTTP_COOKIE')
    return HTTPRedirectResponse(urlto)


def login(request, template_name='auth/login.mako', redirect_field_name='next',
    login_form=None, ssl=False):
    """Main login view for authentication and authorization of remote users
    using cookie-based middleware and secret key. 

    See ``notmm.controllers.auth.AuthkitController`` for more details.
    """
    log.debug("in login view...")
    return HTTPRedirectResponse('/')
    
    # User is already authenticated or is using GET to access the 
    # login screen.
    return direct_to_template(request, template_name, extra_context=data)

def unauthorized(request):
    '''Denies access middleware to unauthorized users'''
    # Only registered accounts may create blog entries
    from notmm.utils.wsgilib import HTTPUnauthorized
    message = '''\
<html>
<head>
 <title>Permission denied</title>
</head>
<body>
<h2>Permission denied</h2>
<p>Please <a href="/session_login/">authenticate</a> first. Anonymous blog
posting is not permitted yet. A valid account is required to post new articles.
</p>
<p>Thanks for your understanding and have fun writing stuff... :)</p>
</body>
</html>
    '''
    return HTTPUnauthorized(message , mimetype='text/html')




#!/usr/bin/env python
# -*- coding: utf-8 -*-

from notmm.utils.urlmap import RegexURLMap, url, include
from notmm.release import VERSION
from notmm.utils.django_settings import LazySettings

from helloworld.forms import LoginForm
from helloworld.login_views import login, logout
settings = LazySettings()
render_to_response = 'notmm.utils.template.direct_to_template'

extra_context = {
    #'settings': settings,
    'local_version': VERSION,
    'media_url': settings.MEDIA_URL 
}

#import pdb; pdb.set_trace()
urlpatterns = RegexURLMap()

urlpatterns.add_routes('',   
   url(r'(?P<path>\w+.(gif|png|jpg|ico))$', \
    'helloworld.views.image_handler', {
        'document_root' : settings.MEDIA_ROOT }
   ),
)

urlpatterns.add_routes('',
   # Frontdoor (a simple generic view)
   url(r'^$|index.html$', render_to_response, {
        'template_name': 'default/index.mako',
        'extra_context': extra_context
   }),
   #url(r'^authorize/', include('oauth2_provider.urls'))
   url(r'^session_login/$', login),
   url(r'^session_logout/$', logout)
   )
#urlpatterns.commit()
#print len(urlpatterns)



from django.db import transaction, DatabaseError
from django.utils.translation import ugettext_lazy as _
from livesettings import * 
from tax.config import TAX_MODULE

TAX_MODULE.add_choice(('tax.modules.area', _('By Country/Area')))
TAX_GROUP = config_get_group('TAX')
        
config_register(
     BooleanValue(TAX_GROUP,
         'TAX_SHIPPING',
         description=_("Tax Shipping?"),
         requires=TAX_MODULE,
         requiresvalue='tax.modules.area',
         default=False)
)

config_register(
     StringValue(TAX_GROUP,
         'TAX_CLASS',
         description=_("TaxClass for shipping"),
         help_text=_("Select a TaxClass that should be applied for shipments."),
         #TODO: [BJK] make this dynamic - doesn't work to have it be preloaded.
         default='Shipping'
     )
)




# Copyright (C) 2005,2006 Johan Dahlin
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

"""
Loader hacks specific for Gazpacho the application.
"""

import gtk

from gazpacho.loader.loader import ObjectBuilder
from gazpacho.loader.custom import ButtonAdapter, DialogAdapter, \
     WindowAdapter, str2bool, adapter_registry
from gazpacho.placeholder import Placeholder
from gazpacho.widgets.base.custom import Custom

class GazpachoButtonAdapter(ButtonAdapter):
    def add(self, parent, child, properties, type):
        # This is a hack, remove when the button saving/loading code
        # is improved
        for cur_child in parent.get_children():
            parent.remove(cur_child)
        ButtonAdapter.add(self, parent, child, properties, type)

class GazpachoWindowAdapter(WindowAdapter):
    object_type = gtk.Window
    def prop_set_visible(self, window, value):
        window.set_data('gazpacho::visible', str2bool(value))

    def prop_set_modal(self, window, value):
        window.set_data('gazpacho::modal', str2bool(value))

adapter_registry.register_adapter(GazpachoWindowAdapter)

# FIXME: This is quite strange, but apparently we need this too.
class GazpachoDialogAdapter(DialogAdapter):
    object_type = gtk.Dialog
    def prop_set_modal(self, window, value):
        window.set_data('gazpacho::modal', str2bool(value))

adapter_registry.register_adapter(GazpachoDialogAdapter)

class GazpachoObjectBuilder(ObjectBuilder):
    def __init__(self, **kwargs):
        self._app = kwargs.pop('app', None)
        kwargs['placeholder'] = self.create_placeholder
        kwargs['custom'] = Custom

        # We want to ignore the domain, otherwise we will end up
        # with a /translated/ interface inside gazpacho itself.
        kwargs['ignore_domain'] = True
        ObjectBuilder.__init__(self, **kwargs)

    def create_placeholder(self, name):
        return Placeholder()

    def show_windows(self):
        pass



from decimal import Decimal
from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _
from l10n.utils import lookup_translation
from product.models import Product, OptionGroup, get_product_quantity_price, get_product_quantity_adjustments
from product.modules.configurable.models import get_all_options
from satchmo_utils.fields import CurrencyField
from satchmo_utils.unique_id import slugify

SATCHMO_PRODUCT=True

def get_product_types():
    return ('CustomProduct',)

class CustomProduct(models.Model):
    """
    Product which must be custom-made or ordered.
    """
    product = models.OneToOneField(Product, verbose_name=_('Product'), primary_key=True)
    downpayment = models.IntegerField(_("Percent Downpayment"), default=20)
    deferred_shipping = models.BooleanField(_('Deferred Shipping'),
        help_text=_('Do not charge shipping at checkout for this item.'),
        default=False)
    option_group = models.ManyToManyField(
        OptionGroup,
        verbose_name=_('Option Group'),
        blank=True)

    def _is_shippable(self):
        return not self.deferred_shipping
    is_shippable = property(fget=_is_shippable)

    def _get_fullPrice(self):
        """
        returns price as a Decimal
        """
        return self.get_qty_price(Decimal('1'))

    unit_price = property(_get_fullPrice)

    def add_template_context(self, context, selected_options, **kwargs):
        """
        Add context for the product template.
        Return the updated context.
        """
        from product.utils import serialize_options

        options = serialize_options(self, selected_options)
        if not 'options' in context:
            context['options'] = options
        else:
            curr = list(context['options'])
            curr.extend(list(options))
            context['options'] = curr

        return context

    def get_qty_price(self, qty, include_discount=True):
        """
        If QTY_DISCOUNT prices are specified, then return the appropriate discount price for
        the specified qty.  Otherwise, return the unit_price
        returns price as a Decimal
        """
        if include_discount:
            price = get_product_quantity_price(self.product, qty)
        else:
            adjustment = get_product_quantity_adjustments(self, qty)
            if adjustment.price is not None:
                price = adjustment.price.price
            else:
                price = None

        if not price and qty == Decimal('1'): # Prevent a recursive loop.
            price = Decimal("0.00")
        elif not price:
            price = self.product._get_fullPrice()

        return price * self.downpayment / 100

    def get_full_price(self, qty=Decimal('1')):
        """
        Return the full price, ignoring the deposit.
        """
        price = get_product_quantity_price(self.product, qty)
        if not price:
            price = self.product.unit_price

        return price

    full_price = property(fget=get_full_price)



    def _get_subtype(self):
        return 'CustomProduct'

    def __unicode__(self):
        return u"CustomProduct: %s" % self.product.name

    def get_valid_options(self):
        """
        Returns all of the valid options
        """
        return get_all_options(self, ids_only=True)

    def save(self, **kwargs):
        if hasattr(self.product,'_sub_types'):
            del self.product._sub_types
        super(CustomProduct, self).save(**kwargs)


    class Meta:
        verbose_name = _('Custom Product')
        verbose_name_plural = _('Custom Products')


class CustomTextField(models.Model):
    """
    A text field to be filled in by a customer.
    """

    name = models.CharField(_('Custom field name'), max_length=40, )
    slug = models.SlugField(_("Slug"), help_text=_("Auto-generated from name if blank"),
        blank=True)
    products = models.ForeignKey(CustomProduct, verbose_name=_('Custom Fields'),
        related_name='custom_text_fields')
    sort_order = models.IntegerField(_("Sort Order"),
        help_text=_("The display order for this group."), default=0)
    price_change = CurrencyField(_("Price Change"), max_digits=14,
        decimal_places=6, blank=True, null=True)

    def save(self, **kwargs):
        if not self.slug:
            self.slug = slugify(self.name, instance=self)
        super(CustomTextField, self).save(**kwargs)

    def translated_name(self, language_code=None):
        return lookup_translation(self, 'name', language_code)

    class Meta:
        ordering = ('sort_order',)
        unique_together = ('slug', 'products')

class CustomTextFieldTranslation(models.Model):
    """A specific language translation for a `CustomTextField`.  This is intended for all descriptions which are not the
    default settings.LANGUAGE.
    """
    customtextfield = models.ForeignKey(CustomTextField, related_name="translations")
    languagecode = models.CharField(_('language'), max_length=10, choices=settings.LANGUAGES)
    name = models.CharField(_("Translated Custom Text Field Name"), max_length=255, )
    version = models.IntegerField(_('version'), default=1)
    active = models.BooleanField(_('active'), default=True)

    class Meta:
        verbose_name = _('CustomTextField Translation')
        verbose_name_plural = _('CustomTextField Translations')
        ordering = ('customtextfield', 'name','languagecode')
        unique_together = ('customtextfield', 'languagecode', 'version')

    def __unicode__(self):
        return u"CustomTextFieldTranslation: [%s] (ver #%i) %s Name: %s" % (self.languagecode, self.version, self.customtextfield, self.name)



from elixir import Entity, ManyToOne, OneToMany, ManyToMany, using_options

class A1(Entity):
    using_options(resolve_root='tests.db1')

    a2s = OneToMany('A2')
    bs = ManyToMany('b.B')

class A2(Entity):
    a1 = ManyToOne('A1')




from decimal import Decimal
from django import forms
from django.contrib.sites.models import Site
from livesettings import config_value
from product.models import Product
from satchmo_store.shop.signals import satchmo_cart_details_query, satchmo_cart_add_complete
from satchmo_utils.numbers import RoundedDecimalError, round_decimal, PositiveRoundedDecimalField
import logging

log = logging.getLogger('shop.forms')

class MultipleProductForm(forms.Form):
    """A form used to add multiple products to the cart."""

    def __init__(self, *args, **kwargs):
        products = kwargs.pop('products', None)

        super(MultipleProductForm, self).__init__(*args, **kwargs)

        if products:
            products = [p for p in products if p.active]
        else:
            products = Product.objects.active_by_site()

        self.slugs = [p.slug for p in products]

        for product in products:
            kw = {
                'label' : product.name,
                'help_text' : product.description,
                'initial' : 0,
                'widget' : forms.TextInput(attrs={'class': 'text'}),
                'required' : False
            }

            qty = PositiveRoundedDecimalField(**kw)
            qty.product = product
            self.fields['qty__%s' % product.slug] = qty

    def save(self, cart, request):
        log.debug('saving');
        self.full_clean()
        cartplaces = config_value('SHOP', 'CART_PRECISION')
        roundfactor = config_value('SHOP', 'CART_ROUNDING')
        site = Site.objects.get_current()

        for name, value in self.cleaned_data.items():
            opt, key = name.split('__')
            log.debug('%s=%s', opt, key)

            quantity = 0
            product = None

            if opt=='qty':
                try:
                    quantity = round_decimal(value, places=cartplaces, roundfactor=roundfactor)
                except RoundedDecimalError:
                    quantity = 0

            if not key in self.slugs:
                log.debug('caught attempt to add product not in the form: %s', key)
            else:
                try:
                    product = Product.objects.get(slug=key, site=site)
                except Product.DoesNotExist:
                    log.debug('caught attempt to add an non-existent product, ignoring: %s', key)

            if product and quantity > Decimal('0'):
                log.debug('Adding %s=%s to cart from MultipleProductForm', key, value)
                details = []
                formdata = request.POST
                satchmo_cart_details_query.send(
                    cart,
                    product=product,
                    quantity=quantity,
                    details=details,
                    request=request,
                    form=formdata
                )
                added_item = cart.add_item(product, number_added=quantity, details=details)
                satchmo_cart_add_complete.send(cart, cart=cart, cartitem=added_item,
                    product=product, request=request, form=formdata)



"""Cookie Storage API

A cookie store to use for storage of session data.

Copyright (c) Etienne Robillard <erob@gthcfoundation.org>
All rights reserved.

"""
__all__ = ['CookieStore']

class CookieStore(object):
    """A generic session storage abstract class

    """

    # Picker (Pickle) protocol emulation (unfinished/not tested)
    #def __getstate__(self):
    #    return
    #def __setstate__(self, state):
    #   return

    def __init__(self, key=None):
        #self._data = threading.local()
        self._data = {}
        self._sessionkey = key

    def get_storage(self):
        return self._data

    _session = property(get_storage)

    def __getattr__(self, value, default=None):
        return self._session.get(value, default)

    def __getitem__(self, value, default=None):
        return self.__getattr__(value, default)

    def __setitem__(self, key, value):
        self._session[key] = value

    def __delitem__(self, value):
        del self._session[value]

    def __contains__(self, value):
        return value in self._session

    # public methods
    def get(self, value, default=None):
        return self.__getitem__(value, default)

    #def get_and_delete_messages(self):
    #    pass

    # iteration procotol
    #def __next__(self):
    #    for item in iter(self):
    #        yield item
    #def __iter__(self):
    #    return self.__dict__.iteritems()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from webhelpers.paginate import Page
from blogengine.config.global_settings import BLOGENGINE
__all__ = ['getpaginator', 'getpageinfo']

# Thanks to the webhelpers.paginate authors for providing a 
# reliable paginator algorithm
def getpaginator(items, settings, paginator=Page):

    return paginator(items, 
        page=BLOGENGINE['PAGE_MAX'], 
        items_per_page=BLOGENGINE['ITEMS_PER_PAGE'])

def getpageinfo(p):
    """Return the next and prev page from the ``p`` instance."""
    if p.page == p.page_count:
        next_page = p.first_page
    else:
        next_page = p.page + 1
    
    if p.page == 1:
        prev_page = p.page
    else:
        prev_page = p.page - 1

    return (next_page, prev_page)




from django.core import urlresolvers
from django.test import TestCase
from django.test.client import Client
import keyedcache
from livesettings import config_get
from satchmo_ext.newsletter import *
from satchmo_ext.newsletter.models import get_contact_or_fake, Subscription
import logging

log = logging.getLogger('test');

class NewsletterTest(TestCase):
    
    def setUp(self):
        cfg = config_get('NEWSLETTER', 'MODULE')
        cfg.update('satchmo_ext.newsletter.simple')
        
    def tearDown(self):
        keyedcache.cache_delete()
        
    def  testNewIsUnsubscribed(self):
        c = get_contact_or_fake('test test', 'testnew@test.com')
        sub = is_subscribed(c)
        self.assertFalse(sub)
        
    def  testExplicitUnsubscribed(self):
        c = get_contact_or_fake('test test', 'explicitno@test.com')
        results = update_subscription(c, False)
        sub = is_subscribed(c)
        self.assertFalse(sub)
        
    def  testExplicitSubscribed(self):
        c = get_contact_or_fake('test test', 'explicityes@test.com')
        results = update_subscription(c, True)
        sub = is_subscribed(c)
        self.assertTrue(sub)

    def testSetReset(self):
        c = get_contact_or_fake('test test', 'setreset@test.com')
        results = update_subscription(c, True)
        sub = is_subscribed(c)
        self.assertTrue(sub)
        
        results = update_subscription(c, False)
        sub = is_subscribed(c)
        self.assertFalse(sub)
        
    def testAttributeSetting(self):
        c = get_contact_or_fake('test test', 'setattr@test.com')
        results = update_subscription(c, True, attributes = {'var1' : 'True'})
        sub = Subscription.objects.get(email='setattr@test.com')
        self.assert_(sub)
        self.assertEqual(sub.attribute_value('var1'), 'True')
        self.assertTrue(sub.subscribed)
        
class NewsletterTestViews(TestCase):
    
    fixtures = ['l10n-data.yaml', 'sample-store-data.yaml']
    
    def setUp(self):
        cfg = config_get('NEWSLETTER', 'MODULE')
        cfg.update('satchmo_ext.newsletter.simple')
        self.client = Client()
        
    def tearDown(self):
        keyedcache.cache_delete()
    
    def  testNewIsSubscribed(self):
        url = urlresolvers.reverse('newsletter_subscribe_ajah')
        c = get_contact_or_fake('test test', 'testsubview@test.com')
        self.assertFalse(is_subscribed(c))
        
        response = self.client.post(url, {
            'full_name' : 'Testy Testerson',
            'email': c.email,
            })
    
        self.assertTrue(is_subscribed(c))

    def  testNewIsUnsubscribed(self):
        c = get_contact_or_fake('test test', 'testunsubview@test.com')
        update_subscription(c, True)
        self.assertTrue(is_subscribed(c))

        url = urlresolvers.reverse('newsletter_unsubscribe_ajah')
        response = self.client.post(url, {
            'full_name' : 'Testy Testerson',
            'email': c.email,
            })

        self.assertFalse(is_subscribed(c))
        
    



from django.core.exceptions import ImproperlyConfigured
from django.conf import settings
from django.utils.translation import get_language, to_locale
from l10n.l10n_settings import get_l10n_setting
import logging

log = logging.getLogger('l10n.utils')

def moneyfmt(val, currency_code=None, wrapcents=''):
    """Formats val according to the currency settings for the desired currency, as set in L10N_SETTINGS"""
    if val is None or val == '':
       val = Decimal('0')
       
    currencies = get_l10n_setting('currency_formats')
    currency = None
    
    if currency_code:
        currency = currencies.get(currency_code, None)
        if not currency:
            log.warn('Could not find currency code definitions for "%s", please look at l10n.l10n_settings for examples.')
    
    if not currency:
        default_currency_code = get_l10n_setting('default_currency', None)

        if not default_currency_code:
            log.fatal("No default currency code set in L10N_SETTINGS")
            raise ImproperlyConfigured("No default currency code set in L10N_SETTINGS")

        if currency_code == default_currency_code:
            raise ImproperlyConfigured("Default currency code '%s' not found in currency_formats in L10N_SETTINGS", currency_code)
        
        return moneyfmt(val, currency_code=default_currency_code, wrapcents=wrapcents)
    
    # here we are assured we have a currency format
    
    if val>=0:
        key = 'positive'
    else:
        val = abs(val)
        key = 'negative'
        
    fmt = currency[key]
    formatted = fmt % { 'val' : val }
    
    sep = currency.get('decimal', '.')
    if sep != '.':
        formatted = formatted.replace('.', sep)

    if wrapcents:
        pos = formatted.rfind(sep)
        if pos>-1:
            pos +=1 
            formatted = u"%s<%s>%s</%s>" % formatted[:pos], wrapcents, formatted[pos:], wrapcents

    return formatted



from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from livesettings import *
from satchmo_store.shop import get_satchmo_setting
from satchmo_utils import is_string_like, load_module

SHIPPING_GROUP = ConfigurationGroup('SHIPPING', _('Shipping Settings'))

SHIPPING_ACTIVE = config_register(MultipleStringValue(SHIPPING_GROUP,
    'MODULES',
    description=_("Active shipping modules"),
    help_text=_("Select the active shipping modules, save and reload to set any module-specific shipping settings."),
    default=["shipping.modules.per"],
    choices=[('shipping.modules.per', _('Per piece'))],
    ordering=0
    ))
    
config_register(
    StringValue(SHIPPING_GROUP,
        'HIDING',
        description = _("Hide if one?"),
        help_text = _("Hide shipping form fields if there is only one choice available?"),
        default='NO',
        ordering=10,
        choices = (
            ('NO', _('No')),
            ('YES', _('Yes')),
            ('DESCRIPTION', _('Show description only'))
        )))
        
config_register(
    BooleanValue(SHIPPING_GROUP,
        'SELECT_CHEAPEST',
        description = _("Select least expensive by default?"),
        default=True,
        ordering=15
        ))


# --- Load default shipping modules.  Ignore import errors, user may have deleted them. ---
# DO NOT ADD 'tiered' or 'no' to this list.  
# 'no' is used internally
# 'Tiered' is special, since it needs to be added as a module.  To enable it,
# just add shipping.modules.tiered to your INSTALLED_APPS
_default_modules = ('canadapost', 'dummy', 'fedex', 'flat', 'per', 'ups', 'usps')

for module in _default_modules:
    try:
    	load_module("shipping.modules.%s.config" % module)
    except ImportError:
        log.debug('Could not load default shipping module configuration: %s', module)

# --- Load any extra shipping modules. ---
extra_shipping = get_satchmo_setting('CUSTOM_SHIPPING_MODULES')

for extra in extra_shipping:
    try:
        load_module("%s.config" % extra)
    except ImportError:
        log.warn('Could not load shipping module configuration: %s' % extra)

class ShippingModuleNotFound(Exception):
    def __init__(key):
        self.key = key

def shipping_methods():
    methods = []
    modules = config_value('SHIPPING', 'MODULES')
    log.debug('Getting shipping methods: %s', modules)
    for m in modules:
        module = load_module(m)
        methods.extend(module.get_methods())
    return methods
    
def shipping_method_by_key(key):
    if key and key != "NoShipping":
        for method in shipping_methods():
            if method.id == key:
                return method
    else:
        import shipping.modules.no.shipper as noship
        method = noship.Shipper()
        
    if method:
        return method
    else:
        raise ShippingModuleNotFound(key)
        

def shipping_choices():
    choices = []
    keys = []
    for method in shipping_methods():
        key = method.id
        label = method.description()
        choices.append((key, label))
    return choices



from decimal import Decimal
from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.core import mail
from django.core.urlresolvers import reverse as url
from django.test import TestCase
from django.test.client import Client
from django.utils.encoding import smart_str
from keyedcache import cache_delete
from l10n.models import Country
from l10n.utils import moneyfmt
from livesettings import config_get
from payment import active_gateways
from product.models import Product
from product.utils import rebuild_pricing, find_auto_discounts
from satchmo_store.contact import CUSTOMER_ID
from satchmo_store.contact.models import *
from satchmo_store.shop import get_satchmo_setting, signals
from satchmo_store.shop.exceptions import CartAddProhibited
from satchmo_store.shop.models import *
from satchmo_utils.templatetags import get_filter_args

import keyedcache

domain = 'http://example.com'
prefix = get_satchmo_setting('SHOP_BASE')
if prefix == '/':
    prefix = ''

def get_step1_post_data(US):
    return {
        'email': 'sometester@example.com',
        'first_name': 'Teddy',
        'last_name' : 'Tester',
        'phone': '456-123-5555',
        'street1': '8299 Some Street',
        'city': 'Springfield',
        'state': 'MO',
        'postal_code': '81122',
        'country': US.pk,
        'ship_street1': '1011 Some Other Street',
        'ship_city': 'Springfield',
        'ship_state': 'MO',
        'ship_postal_code': '81123',
        'paymentmethod': 'PAYMENT_DUMMY',
        'copy_address' : True
        }

def make_order_payment(order, paytype=None, amount=None):
    if not paytype:
        paytype = active_gateways()[0][0]

    if not amount:
        amount = order.balance

    pmt = OrderPayment(order=order, payment=paytype.upper(), amount=amount)
    pmt.save()
    return pmt

class ShopTest(TestCase):
    fixtures = ['l10n-data.yaml', 'sample-store-data.yaml', 'products.yaml', 'test-config.yaml', 'initial_data.yaml']

    def setUp(self):
        # Every test needs a client
        cache_delete()
        self.client = Client()
        self.US = Country.objects.get(iso2_code__iexact = "US")
        rebuild_pricing()

    def tearDown(self):
        cache_delete()

    def test_main_page(self):
        """
        Look at the main page
        """
        response = self.client.get(prefix+'/')

        # Check that the rendered context contains 4 products
        self.assertContains(response, '<div class = "productImage">',
                            count=4, status_code=200)

    def test_contact_form(self):
        """
        Validate the contact form works
        """

        response = self.client.get(prefix+'/contact/')
        self.assertContains(response, '<h3>Contact Information</h3>',
                            count=1, status_code=200)
        response = self.client.post(prefix+'/contact/', {'name': 'Test Runner',
                              'sender': 'Someone@testrunner.com',
                              'subject': 'A question to test',
                              'inquiry': 'General Question',
                              'contents': 'A lot of info goes here.'})
        self.assertRedirects(response, prefix + '/contact/thankyou/',
            status_code=302, target_status_code=200)
        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].subject, 'A question to test')

    def test_new_account(self):
        """
        Validate account creation process
        """
        shop_config = Config.objects.get_current()
        subject = u"Welcome to %s" % shop_config.store_name
        response = self.client.get('/accounts/register/')
        self.assertContains(response, "Please Enter Your Account Information",
                            count=1, status_code=200)
        response = self.client.post('/accounts/register/', {'email': 'someone@test.com',
                                    'first_name': 'Paul',
                                    'last_name' : 'Test',
                                    'password1' : 'pass1',
                                    'password2' : 'pass1',
                                    'newsletter': '0'})
        self.assertRedirects(response, '/accounts/register/complete/',
            status_code=302, target_status_code=200)
        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].subject, subject)

        response = self.client.get('/accounts/')
        self.assertContains(response, "Welcome, Paul Test.", count=1, status_code=200)
        response = self.client.get('/accounts/logout/')

    def test_cart_adding(self, retest=False):
        """
        Validate we can add some items to the cart
        """
        response = self.client.get(prefix+'/product/dj-rocks/')
        if not retest:
            self.assertContains(response, "Django Rocks shirt", count=2, status_code=200)
        response = self.client.post(prefix+'/cart/add/', { "productname" : "dj-rocks",
                                                      "1" : "L",
                                                      "2" : "BL",
                                                      "quantity" : 2})
        if not retest:
            self.assertRedirects(response, prefix + '/cart/',
                status_code=302, target_status_code=200)
        response = self.client.get(prefix+'/cart/')
        self.assertContains(response, "Django Rocks shirt (Large/Blue)", count=1, status_code=200)

    def test_cart_adding_errors_nonexistent(self):
        """
        Test proper error reporting when attempting to add items to the cart.
        """

        # Attempting to add a nonexistent product should result in a 404 error.
        response = self.client.post(prefix + '/cart/add/',
            {'productname': 'nonexistent-product', 'quantity': '1'})
        self.assertContains(response, "The product you have requested does not exist.", count=1, status_code=404)

    def test_cart_adding_errors_inactive(self):
        # You should not be able to add a product that is inactive.
        py_shirt = Product.objects.get(slug='PY-Rocks')
        py_shirt.active = False
        py_shirt.save()
        response = self.client.post(prefix + '/cart/add/',
            {'productname': 'PY-Rocks', 'quantity': '1'})
        self.assertContains(response, "That product is not available at the moment.", count=1, status_code=404)

    def test_cart_adding_errors_invalid_qty(self):
        # You should not be able to add a product with a non-valid decimal quantity.
        response = self.client.post(prefix + '/cart/add/',
            {'productname': 'neat-book', '3': 'soft', 'quantity': '1.5a'})

        err = self.client.session.get('ERRORS')
        url = prefix + '/product/neat-book-soft/'
        self.assertRedirects(response, url, status_code=302, target_status_code=200)
        self.assertEqual(err, "Invalid quantity.")

    def test_cart_adding_errors_less_zero(self):
        # You should not be able to add a product with a quantity less than zero.
        response = self.client.post(prefix + '/cart/add/',
            {'productname': 'neat-book', '3': 'soft', 'quantity': '0'})

        err = self.client.session.get('ERRORS')
        url = prefix + '/product/neat-book-soft/'
        self.assertRedirects(response, url, status_code=302, target_status_code=200)
        self.assertEqual(err, "Please enter a positive number.")

    def test_cart_adding_errors_out_of_stock(self):
        # If no_stock_checkout is False, you should not be able to order a
        # product that is out of stock.
        setting = config_get('PRODUCT','NO_STOCK_CHECKOUT')
        setting.update(False)
        response = self.client.post(prefix + '/cart/add/',
            {'productname': 'neat-book', '3': 'soft', 'quantity': '1'})

        err = self.client.session.get('ERRORS')
        url = prefix + '/product/neat-book-soft/'
        self.assertRedirects(response, url, status_code=302, target_status_code=200)
        self.assertEqual(err, "'A really neat book (Soft cover)' is out of stock.")

    def test_product(self):
        # Test for an easily missed reversion. When you lookup a productvariation product then
        # you should get the page of the parent configurableproduct, but with the options for
        # that variation already selected
        response = self.client.get(prefix+'/product/neat-book-soft/')
        self.assertContains(response, 'option value="soft" selected="selected"')
        amount = moneyfmt(Decimal('5.00'))
        self.assertContains(response, smart_str(amount))

    def test_orphaned_product(self):
        """
        Get the page of a Product that is not in a Category.
        """
        Product.objects.create(name="Orphaned Product", slug="orphaned-product", site=Site.objects.get_current())
        response = self.client.get(prefix + '/product/orphaned-product/')
        self.assertContains(response, 'Orphaned Product')
        self.assertContains(response, 'Software')

    def test_get_price(self):
        """
        Get the price and productname of a ProductVariation.
        """
        response = self.client.get(prefix+'/product/dj-rocks/')
        self.assertContains(response, "Django Rocks shirt", count=2, status_code=200)

        # this tests the unmolested price from the ConfigurableProduct, and
        # makes sure we get a good productname back for the ProductVariation
        response = self.client.post(prefix+'/product/dj-rocks/prices/', {"1" : "S",
                                                      "2" : "B",
                                                      "quantity" : '1'})
        content = response.content.split(',')
        self.assertEquals(content[0], '["dj-rocks-s-b"')
        self.assert_(content[1].endswith('20.00"]'))

        # This tests the option price_change feature, and again the productname
        response = self.client.post(prefix+'/product/dj-rocks/prices/', {"1" : "L",
                                                      "2" : "BL",
                                                      "quantity" : '2'})
        content = response.content.split(',')
        self.assertEqual(content[0], '["dj-rocks-l-bl"')
        self.assert_(content[1].endswith('23.00"]'))

    def test_cart_removing(self):
        """
        Validate we can remove an item
        """
        setting = config_get('PRODUCT','NO_STOCK_CHECKOUT')
        setting.update(True)

        self.test_cart_adding(retest=True)
        response = self.client.post(prefix + '/cart/remove/', {'cartitem': '1'})
        #self.assertRedirects(response, prefix + '/cart/',
        #    status_code=302, target_status_code=200)
        response = self.client.get(prefix+'/cart/')
        self.assertContains(response, "Your cart is empty.", count=1, status_code=200)

    def test_checkout(self):
        """
        Run through a full checkout process
        """
        cache_delete()
        tax = config_get('TAX','MODULE')
        tax.update('tax.modules.percent')
        pcnt = config_get('TAX', 'PERCENT')
        pcnt.update('10')
        shp = config_get('TAX', 'TAX_SHIPPING')
        shp.update(False)

        self.test_cart_adding()
        response = self.client.post(url('satchmo_checkout-step1'), get_step1_post_data(self.US))
        self.assertRedirects(response, url('DUMMY_satchmo_checkout-step2'),
            status_code=302, target_status_code=200)
        data = {
            'credit_type': 'Visa',
            'credit_number': '4485079141095836',
            'month_expires': '1',
            'year_expires': '2014',
            'ccv': '552',
            'shipping': 'FlatRate'}
        response = self.client.post(url('DUMMY_satchmo_checkout-step2'), data)
        self.assertRedirects(response, url('DUMMY_satchmo_checkout-step3'),
            status_code=302, target_status_code=200)
        response = self.client.get(url('DUMMY_satchmo_checkout-step3'))
        amount = smart_str('Shipping + ' + moneyfmt(Decimal('4.00')))
        self.assertContains(response, amount, count=1, status_code=200)

        amount = smart_str('Tax + ' + moneyfmt(Decimal('4.60')))
        self.assertContains(response, amount, count=1, status_code=200)

        amount = smart_str('Total = ' + moneyfmt(Decimal('54.60')))
        self.assertContains(response, amount, count=1, status_code=200)

        response = self.client.post(url('DUMMY_satchmo_checkout-step3'), {'process' : 'True'})
        self.assertRedirects(response, url('DUMMY_satchmo_checkout-success'),
            status_code=302, target_status_code=200)
        self.assertEqual(len(mail.outbox), 1)

        # Log in as a superuser
        user = User.objects.create_user('fredsu', 'fred@root.org', 'passwd')
        user.is_staff = True
        user.is_superuser = True
        user.save()
        self.client.login(username='fredsu', password='passwd')

        # Test pdf generation
        response = self.client.get('/admin/print/invoice/1/')
        self.assertContains(response, 'reportlab', status_code=200)
        response = self.client.get('/admin/print/packingslip/1/')
        self.assertContains(response, 'reportlab', status_code=200)
        response = self.client.get('/admin/print/shippinglabel/1/')
        self.assertContains(response, 'reportlab', status_code=200)

    def test_contact_login(self):
        """Check that when a user logs in, the user's existing Contact will be
        used.
        """
        user = User.objects.create_user('teddy', 'sometester@example.com', 'guz90tyc')
        contact = Contact.objects.create(user=user, first_name="Teddy",
            last_name="Tester")
        self.client.login(username='teddy', password='guz90tyc')
        self.test_cart_adding()
        response = self.client.get(url('satchmo_checkout-step1'))
        self.assertContains(response, "Teddy", status_code=200)

    def test_registration_keeps_contact(self):
        """Check that if a user creates a Contact and later registers,
        the existing Contact will be attached to the User.
        """
        self.test_cart_adding()
        response = self.client.post(prefix + '/checkout/', get_step1_post_data(self.US))
        self.assert_(self.client.session.get(CUSTOMER_ID) is not None)
        response = self.client.get('/accounts/register/')
        self.assertContains(response, "Teddy", status_code=200)
        origcontact = Contact.objects.get(email="sometester@example.com")
        self.assert_(origcontact)
        data = {
            'email': 'sometester@example.com',
            'first_name': 'Teddy',
            'last_name': 'Tester',
            'password1': 'guz90tyc',
            'password2': 'guz90tyc',
            'newsletter': '0'}
        response = self.client.post('/accounts/register/', data)
        self.assertRedirects(response, '/accounts/register/complete/',
            status_code=302, target_status_code=200)
        user = User.objects.get(email="sometester@example.com")
        contact = user.contact_set.get()
        self.assertEqual(contact, origcontact)

    def test_contact_email_security(self):
        """
        Validate that we can't create a new contact with an existing contact's email address.
        Ticket #233
        """
        self.test_new_account()
        response = self.client.get('/accounts/register/')
        init_data = {
            'email': 'somenewtester@example.com',
            'first_name': 'New',
            'last_name': 'Tester',
            'password1': 'new123pass',
            'password2': 'new123pass',
            'newsletter': '0'}
        response = self.client.post('/accounts/register/', init_data)
        self.assertRedirects(response, '/accounts/register/complete/',
            status_code=302, target_status_code=200)
        response = self.client.get('/accounts/update')
        full_data = {
            'first_name': 'New',
            'last_name': 'Tester',
            'email': 'someone@test.com',
            'phone': '901-881-1230',
            'street1': '8 First Street',
            'city': 'Littleton',
            'state': 'MA',
            'postal_code': '01229',
            'country': self.US.pk,
            'ship_street1': '11 Easy Street',
            'ship_city': 'Littleton',
            'ship_state': 'MA',
            'ship_postal_code': '01229',
        }
        response = self.client.post('/accounts/update/', full_data)
        self.assertContains(response,"That email address is already in use", status_code=200)
        full_data['email'] = 'somenewtester@example.com'
        response = self.client.post('/accounts/update/', full_data)
        response = self.client.get('/accounts/')
        self.assertContains(response,"Email: somenewtester@example.com")

    def test_contact_attaches_to_user(self):
        """Check that if a User registers and later creates a Contact, the
        Contact will be attached to the existing User.
        """
        user = User.objects.create_user('teddy', 'sometester@example.com', 'guz90tyc')
        self.assertEqual(user.contact_set.count(), 0)
        self.client.login(username='teddy', password='guz90tyc')
        self.test_cart_adding()
        response = self.client.post(prefix + '/checkout/', get_step1_post_data(self.US))
        self.assertEqual(user.contact_set.count(), 1)

    def test_logout(self):
        """The logout view should remove the user and contact id from the
        session.
        """
        user = User.objects.create_user('teddy', 'sometester@example.com', 'guz90tyc')
        self.client.login(username='teddy', password='guz90tyc')
        response = self.client.get('/accounts/') # test logged in status
        self.assertContains(response, "the user you've logged in as doesn't have any contact information.", status_code=200)
        self.test_cart_adding()
        self.client.post(prefix + '/checkout/', get_step1_post_data(self.US))
        self.assert_(self.client.session.get(CUSTOMER_ID) is not None)
        response = self.client.get('/accounts/logout/?next=%s' % (prefix+'/checkout/'))
        self.assertRedirects(response, prefix + '/checkout/',
            status_code=302, target_status_code=200)
        self.assert_(self.client.session.get(CUSTOMER_ID) is None)
        response = self.client.get('/accounts/') # test logged in status
        self.assertRedirects(response, '/accounts/login/?next=/accounts/',
            status_code=302, target_status_code=200)

    def test_search(self):
        """
        Do some basic searches to make sure it all works as expected
        """
        response = self.client.get(prefix+'/search/', {'keywords':'python'})
        self.assertContains(response, "Python Rocks shirt", count=1)
        response = self.client.get(prefix+'/search/', {'keywords':'django+book'})
        self.assertContains(response, "Nothing found")
        response = self.client.get(prefix+'/search/', {'keywords':'shirt'})
        self.assertContains(response, "Shirts", count=2)
        self.assertContains(response, "Short Sleeve", count=2)
        self.assertContains(response, "Django Rocks shirt", count=10)
        self.assertContains(response, "Python Rocks shirt", count=1)

class AdminTest(TestCase):
    fixtures = ['l10n-data.yaml', 'sample-store-data.yaml', 'products.yaml', 'initial_data.yaml']

    def setUp(self):
        self.client = Client()
        user = User.objects.create_user('fredsu', 'fred@root.org', 'passwd')
        user.is_staff = True
        user.is_superuser = True
        user.save()
        self.client.login(username='fredsu', password='passwd')

    def tearDown(self):
        cache_delete()

    def test_index(self):
        response = self.client.get('/admin/')
        self.assertContains(response, "contact/contact/", status_code=200)

    #def test_product(self):
        response = self.client.get('/admin/product/product/1/')
        self.assertContains(response, "Django Rocks shirt", status_code=200)

    #def test_configurableproduct(self):
        response = self.client.get('/admin/configurable/configurableproduct/1/')
        self.assertContains(response, "Small, Black", status_code=200)

    #def test_productimage_list(self):
        # response = self.client.get('/admin/product/productimage/')
        # self.assertContains(response, "Photo Not Available", status_code=200)

    #def test_productimage(self):
        # response = self.client.get('/admin/product/productimage/1/')
        # self.assertContains(response, "Photo Not Available", status_code=200)

class FilterUtilTest(TestCase):
    """Test the templatetags util class"""

    def tearDown(self):
        cache_delete()

    def test_simple_get_args(self):
        args, kwargs = get_filter_args('one=1,two=2')
        self.assertEqual(len(args), 0)

        self.assertEqual(kwargs['one'], '1')

        self.assertEqual(kwargs['two'], '2')

    def test_extended_get_args(self):
        args, kwargs = get_filter_args('test,one=1,two=2')
        self.assertEqual(args[0], 'test')

        self.assertEqual(kwargs['one'], '1')

        self.assertEqual(kwargs['two'], '2')

    def test_numerical_get_args(self):
        args, kwargs = get_filter_args('test,one=1,two=2', (), ('one','two'))
        self.assertEqual(args[0], 'test')

        self.assertEqual(kwargs['one'], 1)

        self.assertEqual(kwargs['two'], 2)

    def test_keystrip_get_args(self):
        args, kwargs = get_filter_args('test,one=1,two=2', ('one'), ('one'))
        self.assertEqual(args[0], 'test')

        self.assertEqual(kwargs['one'], 1)

        self.assertFalse('two' in kwargs)


    def test_stripquotes_get_args(self):
        args, kwargs = get_filter_args('"test",one="test",two=2', stripquotes=True)
        self.assertEqual(args[0], 'test')

        self.assertEqual(kwargs['one'], 'test')

        self.assertEqual(kwargs['two'], '2')

        args, kwargs = get_filter_args('"test",one="test",two=2', stripquotes=False)
        self.assertEqual(args[0], '"test"')

        self.assertEqual(kwargs['one'], '"test"')

class CartTest(TestCase):
    fixtures = ['l10n-data.yaml', 'sample-store-data.yaml', 'products.yaml', 'test-config.yaml']

    def tearDown(self):
        cache_delete()
        rebuild_pricing()

    def test_line_cost(self):
        p = Product.objects.get(slug__iexact='dj-rocks')
        lb = Product.objects.get(slug__iexact='dj-rocks-l-bl')
        sb = Product.objects.get(slug__iexact='dj-rocks-s-b')

        cart = Cart(site=Site.objects.get_current())
        cart.save()
        cart.add_item(sb, 1)
        self.assertEqual(cart.numItems, 1)
        self.assertEqual(cart.total, Decimal("20.00"))

        cart.add_item(lb, 1)
        self.assertEqual(cart.numItems, 2)
        items = list(cart.cartitem_set.all())
        item1 = items[0]
        item2 = items[1]
        self.assertEqual(item1.unit_price, Decimal("20.00"))
        self.assertEqual(item2.unit_price, Decimal("23.00"))
        self.assertEqual(cart.total, Decimal("43.00"))

class ConfigTest(TestCase):
    fixtures = ['l10n-data.yaml', 'sample-store-data.yaml', 'test-config.yaml']

    def tearDown(self):
        cache_delete()

    def test_base_url(self):
        config = Config.objects.get_current()
        self.assertEquals(config.base_url, domain)

class DiscountAmountTest(TestCase):
    fixtures = ['l10n-data.yaml', 'test_discount.yaml', 'initial_data.yaml']

    def setUp(self):
        self.US = Country.objects.get(iso2_code__iexact = 'US')
        self.site = Site.objects.get_current()
        tax = config_get('TAX','MODULE')
        tax.update('tax.modules.no')
        c = Contact(first_name="Jim", last_name="Tester",
            role=ContactRole.objects.get(pk='Customer'), email="Jim@JimWorld.com")
        c.save()
        ad = AddressBook(contact=c, description="home",
            street1 = "test", state="OR", city="Portland",
            country = self.US, is_default_shipping=True,
            is_default_billing=True)
        ad.save()
        o = Order(contact=c, shipping_cost=Decimal('6.00'), site=self.site)
        o.save()
        small = Order(contact=c, shipping_cost=Decimal('6.00'), site=self.site)
        small.save()

        p = Product.objects.get(slug='neat-book-soft')
        price = p.unit_price
        item1 = OrderItem(order=o, product=p, quantity='1',
            unit_price=price, line_item_price=price)
        item1.save()

        item1s = OrderItem(order=small, product=p, quantity='1',
            unit_price=price, line_item_price=price)
        item1s.save()

        p = Product.objects.get(slug='neat-book-hard')
        price = p.unit_price
        item2 = OrderItem(order=o, product=p, quantity='1',
            unit_price=price, line_item_price=price)
        item2.save()
        self.order = o
        self.small = small

        rebuild_pricing()

    def tearDown(self):
        cache_delete()

    def testBase(self):
        """Check base prices"""
        self.order.recalculate_total(save=False)
        price = self.order.total
        shipcost = self.order.shipping_cost
        shiptotal = self.order.shipping_sub_total
        sub_total = self.order.sub_total
        discount = self.order.discount

        self.assertEqual(price, Decimal('17.00'))
        self.assertEqual(sub_total, Decimal('11.00'))
        self.assertEqual(shipcost, Decimal('6.00'))
        self.assertEqual(shiptotal, Decimal('6.00'))
        self.assertEqual(discount, Decimal('0.00'))

        self.small.recalculate_total(save=False)
        price = self.small.total
        shipcost = self.small.shipping_cost
        shiptotal = self.small.shipping_sub_total
        sub_total = self.small.sub_total
        discount = self.small.discount

        self.assertEqual(price, Decimal('12.00'))
        self.assertEqual(sub_total, Decimal('6.00'))
        self.assertEqual(shipcost, Decimal('6.00'))
        self.assertEqual(shiptotal, Decimal('6.00'))
        self.assertEqual(discount, Decimal('0.00'))


    def testApplyAmountSimple(self):
        """Check straight amount discount."""
        self.order.discount_code="test10"
        self.order.recalculate_total(save=False)
        sub_total = self.order.sub_total
        price = self.order.total
        shipcost = self.order.shipping_cost
        shiptotal = self.order.shipping_sub_total
        discount = self.order.discount

        self.assertEqual(sub_total, Decimal('11.00'))
        self.assertEqual(price, Decimal('7.00'))
        self.assertEqual(shipcost, Decimal('6.00'))
        self.assertEqual(shiptotal, Decimal('6.00'))
        self.assertEqual(discount, Decimal('10.00'))


    def testApplySmallAmountSimple(self):
        """Check small straight amount discount."""
        self.small.discount_code="test10"
        self.small.recalculate_total(save=False)
        sub_total = self.small.sub_total
        price = self.small.total
        shipcost = self.small.shipping_cost
        shiptotal = self.small.shipping_sub_total
        discount = self.small.discount

        self.assertEqual(sub_total, Decimal('6.00'))
        self.assertEqual(price, Decimal('6.00'))
        self.assertEqual(shipcost, Decimal('6.00'))
        self.assertEqual(shiptotal, Decimal('6.00'))
        self.assertEqual(discount, Decimal('6.00'))

    def testApplyAmountShip(self):
        """Check amount discount w/ship."""
        self.order.discount_code="test10ship"
        self.order.recalculate_total(save=False)
        sub_total = self.order.sub_total
        price = self.order.total
        shipcost = self.order.shipping_cost
        shiptotal = self.order.shipping_sub_total
        discount = self.order.discount

        self.assertEqual(sub_total, Decimal('11.00'))
        self.assertEqual(price, Decimal('7.00'))
        self.assertEqual(shipcost, Decimal('6.00'))
        self.assertEqual(shiptotal, Decimal('2.67'))
        self.assertEqual(discount, Decimal('10.00'))

    def testApplySmallAmountShip(self):
        """Check small amount discount w/ship."""
        self.small.discount_code="test10ship"
        self.small.recalculate_total(save=False)
        sub_total = self.small.sub_total
        price = self.small.total
        shipcost = self.small.shipping_cost
        shiptotal = self.small.shipping_sub_total
        discount = self.small.discount

        self.assertEqual(sub_total, Decimal('6.00'))
        self.assertEqual(price, Decimal('2.00'))
        self.assertEqual(shipcost, Decimal('6.00'))
        self.assertEqual(shiptotal, Decimal('1.00'))
        self.assertEqual(discount, Decimal('10.00'))

    def testApplyAmountFreeShip(self):
        """Check amount discount w/free ship."""
        self.order.discount_code="test10freeship"
        self.order.recalculate_total(save=True)
        sub_total = self.order.sub_total
        price = self.order.total
        shipcost = self.order.shipping_cost
        shiptotal = self.order.shipping_sub_total
        discount = self.order.discount
        ship_discount = self.order.shipping_discount

        self.assertEqual(sub_total, Decimal('11.00'))
        self.assertEqual(price, Decimal('1.00'))
        self.assertEqual(shipcost, Decimal('6.00'))
        self.assertEqual(shiptotal, Decimal('0.00'))
        self.assertEqual(ship_discount, Decimal('6.00'))
        self.assertEqual(discount, Decimal('16.00'))

    def testApplySmallAmountFreeShip(self):
        """Check amount discount w/free ship."""
        self.small.discount_code="test10freeship"
        self.small.recalculate_total(save=True)
        sub_total = self.small.sub_total
        price = self.small.total
        shipcost = self.small.shipping_cost
        shiptotal = self.small.shipping_sub_total
        discount = self.small.discount

        self.assertEqual(sub_total, Decimal('6.00'))
        self.assertEqual(price, Decimal('0.00'))
        self.assertEqual(shipcost, Decimal('6.00'))
        self.assertEqual(price, Decimal('0.00'))
        self.assertEqual(shiptotal, Decimal('0.00'))
        self.assertEqual(discount, Decimal('12.00'))


    def testApplyPercentSimple(self):
        self.order.discount_code="test20"
        self.order.recalculate_total(save=False)
        sub_total = self.order.sub_total
        price = self.order.total
        shipcost = self.order.shipping_cost
        shiptotal = self.order.shipping_sub_total
        discount = self.order.discount

        self.assertEqual(sub_total, Decimal('11.00'))
        self.assertEqual(price, Decimal('14.80'))
        self.assertEqual(shipcost, Decimal('6.00'))
        self.assertEqual(shiptotal, Decimal('6.00'))
        self.assertEqual(discount, Decimal('2.20'))

    def testApplySmallPercentSimple(self):
        self.small.discount_code="test20"
        self.small.recalculate_total(save=False)
        sub_total = self.small.sub_total
        price = self.small.total
        shipcost = self.small.shipping_cost
        shiptotal = self.small.shipping_sub_total
        discount = self.small.discount

        self.assertEqual(sub_total, Decimal('6.00'))
        self.assertEqual(price, Decimal('10.80'))
        self.assertEqual(shipcost, Decimal('6.00'))
        self.assertEqual(shiptotal, Decimal('6.00'))
        self.assertEqual(discount, Decimal('1.20'))

    def testApplyPercentShip(self):
        self.order.discount_code="test20ship"
        self.order.recalculate_total(save=False)
        sub_total = self.order.sub_total
        price = self.order.total
        shipcost = self.order.shipping_cost
        shiptotal = self.order.shipping_sub_total
        discount = self.order.discount

        self.assertEqual(sub_total, Decimal('11.00'))
        self.assertEqual(price, Decimal('13.60'))
        self.assertEqual(shipcost, Decimal('6.00'))
        self.assertEqual(shiptotal, Decimal('4.80'))
        self.assertEqual(discount, Decimal('3.40'))

    def testApplySmallPercentShip(self):
        self.small.discount_code="test20ship"
        self.small.recalculate_total(save=False)
        sub_total = self.small.sub_total
        price = self.small.total
        shipcost = self.small.shipping_cost
        shiptotal = self.small.shipping_sub_total
        discount = self.small.discount

        self.assertEqual(sub_total, Decimal('6.00'))
        self.assertEqual(price, Decimal('9.60'))
        self.assertEqual(shipcost, Decimal('6.00'))
        self.assertEqual(shiptotal, Decimal('4.80'))
        self.assertEqual(discount, Decimal('2.40'))

    def testApplySmallPercentFreeShip(self):
        self.small.discount_code="test20freeship"
        self.small.recalculate_total(save=False)
        sub_total = self.small.sub_total
        price = self.small.total
        shipcost = self.small.shipping_cost
        shiptotal = self.small.shipping_sub_total
        discount = self.small.discount

        self.assertEqual(sub_total, Decimal('6.00'))
        self.assertEqual(price, Decimal('4.80'))
        self.assertEqual(shipcost, Decimal('6.00'))
        self.assertEqual(shiptotal, Decimal('0.00'))
        self.assertEqual(discount, Decimal('7.20'))

    def testApplyPercentProducts(self):
        """
        Check that a percentage discount applies only to products specified in
        valid_products.
        """
        #
        # The discount "test10-product" applies only to 'neat-book-soft'.
        #
        # +-----------------+-----------+----------------------+
        # | Product         | Price ($) | Discounted price ($) |
        # +=================+===========+======================+
        # | 'neat-book-hard'|     $5    |      $5              |
        # +-----------------+-----------+----------------------+
        # | 'neat-book-soft'|     $6    |      $5.40           |
        # +-----------------+-----------+----------------------+
        # |          Total: |    $11    |     $10.40           |
        # +-----------------+-----------+----------------------+
        #

        discount_obj = Discount.objects.by_code("test10-product")
        products = {}
        for orderitem in self.order.orderitem_set.all():
            products[orderitem.product.slug] = orderitem.product
        self.assertFalse(discount_obj.valid_for_product(products['neat-book-hard']))
        self.assertTrue(discount_obj.valid_for_product(products['neat-book-soft']))

        self.order.discount_code=discount_obj.code
        self.order.recalculate_total(save=False)
        sub_total = self.order.sub_total
        price = self.order.total
        shipcost = self.order.shipping_cost
        shiptotal = self.order.shipping_sub_total
        discount = self.order.discount

        self.assertEqual(sub_total, Decimal('11.00'))
        self.assertEqual(price, Decimal('16.40'))
        self.assertEqual(shipcost, Decimal('6.00'))
        self.assertEqual(shiptotal, Decimal('6.00'))
        self.assertEqual(discount, Decimal('0.60'))

    def testRetrieveAutoDiscounts(self):
        products = [i.product for i in self.order.orderitem_set.all()]
        discounts = find_auto_discounts(products)

        self.assertEqual(
            set([d.code for d in discounts]),
            set(['test20-auto', 'test20-auto-all']))
        self.assertEqual(
            [d.percentage for d in discounts],
            [Decimal('20.0'), Decimal('20.0')])

    def make_order_payment(order, paytype=None, amount=None):
        if not paytype:
            paytype = active_gateways()[0][0]

        if not amount:
            amount = order.balance

        pmt = OrderPayment(order=order, payment=paytype.upper(), amount=amount)
        pmt.save()
        return pmt

def make_test_order(country, state, include_non_taxed=False, site=None, price=None, quantity=5):
    if not site:
        site = Site.objects.get_current()
    c = Contact(first_name="Tax", last_name="Tester",
        role=ContactRole.objects.get(pk='Customer'), email="tax@example.com")
    c.save()
    if not isinstance(country, Country):
        country = Country.objects.get(iso2_code__iexact = country)

    ad = AddressBook(contact=c, description="home",
        street1 = "test", state=state, city="Portland",
        country = country, is_default_shipping=True,
        is_default_billing=True)
    ad.save()
    o = Order(contact=c, shipping_cost=Decimal('10.00'), site = site)
    o.save()

    p = Product.objects.get(slug='dj-rocks-s-b')
    if not price:
        price = p.unit_price
    item1 = OrderItem(order=o, product=p, quantity=quantity,
        unit_price=price, line_item_price=price*quantity)
    item1.save()

    if include_non_taxed:
        p = Product.objects.get(slug='neat-book-hard')
        price = p.unit_price
        item2 = OrderItem(order=o, product=p, quantity='1',
            unit_price=price, line_item_price=price)
        item2.save()

    return o

class OrderTest(TestCase):
    fixtures = ['l10n-data.yaml', 'test_multishop.yaml', 'products.yaml', 'initial_data.yaml']

    def setUp(self):
        keyedcache.cache_delete()
        self.US = Country.objects.get(iso2_code__iexact='US')

    def tearDown(self):
        cache_delete()

    def testBalanceMethods(self):
        order = make_test_order(self.US, '', include_non_taxed=True)
        order.recalculate_total(save=False)
        price = order.total
        subtotal = order.sub_total

        self.assertEqual(subtotal, Decimal('105.00'))
        self.assertEqual(price, Decimal('115.00'))
        self.assertEqual(order.balance, price)

        paytype = active_gateways()[0][0].upper()

        pmt = OrderPayment(order = order, payment=paytype, amount=Decimal("5.00"))
        pmt.save()

        self.assertEqual(order.balance, Decimal("110.00"))
        self.assertEqual(order.balance_paid, Decimal("5.00"))

        self.assert_(order.is_partially_paid)

        pmt = OrderPayment(order = order, payment=paytype, amount=Decimal("110.00"))
        pmt.save()

        self.assertEqual(order.balance, Decimal("0.00"))
        self.assertEqual(order.is_partially_paid, False)
        self.assert_(order.paid_in_full)

    def testSmallPayment(self):
        order = make_test_order(self.US, '', include_non_taxed=True)
        order.recalculate_total(save=False)
        price = order.total
        subtotal = order.sub_total

        paytype = active_gateways()[0][0].upper()
        pmt = OrderPayment(order = order, payment=paytype, amount=Decimal("0.000001"))
        pmt.save()

        self.assert_(order.is_partially_paid)

class QuickOrderTest(TestCase):
    """Test quickorder sheet."""
    fixtures = ['l10n-data.yaml', 'sample-store-data.yaml', 'products.yaml', 'test-config.yaml', 'initial_data.yaml']

    def setUp(self):
        keyedcache.cache_delete()
        self.US = Country.objects.get(iso2_code__iexact='US')

    def tearDown(self):
        cache_delete()

    def testQuickOrderAdd(self):
        """Test adding multiple products at once to cart."""
        response = self.client.get(url('satchmo_quick_order'))
        self.assertContains(response, "Django Rocks shirt (Large/Black)", status_code=200)
        self.assertContains(response, "Python Rocks shirt", status_code=200)
        response = self.client.post(url('satchmo_quick_order'), {
            "qty__dj-rocks-l-b" : "2",
            "qty__PY-Rocks" : "1",
        })
        #print response
        self.assertRedirects(response, url('satchmo_cart'),
            status_code=302, target_status_code=200)
        response = self.client.get(prefix+'/cart/')
        self.assertContains(response, "Django Rocks shirt (Large/Black)" , status_code=200)
        self.assertContains(response, "Python Rocks shirt")
        cart = Cart.objects.latest('id')
        self.assertEqual(cart.numItems, 3)
        products = [(item.product.slug, item.quantity) for item in cart.cartitem_set.all()]
        products.sort()
        self.assertEqual(len(products), 2)
        self.assertEqual(products[0][0], 'PY-Rocks')
        self.assertEqual(products[0][1], Decimal(1))
        self.assertEqual(products[1][0], 'dj-rocks-l-b')
        self.assertEqual(products[1][1], Decimal(2))

def vetoAllListener(sender, vetoes={}, **kwargs):
    raise CartAddProhibited(None, "No")

class SignalTest(TestCase):
    fixtures = ['l10n-data.yaml', 'test_multishop.yaml', 'products.yaml']

    def setUp(self):
        keyedcache.cache_delete()
        signals.satchmo_cart_add_verify.connect(vetoAllListener)
        self.US = Country.objects.get(iso2_code__iexact='US')

    def tearDown(self):
        cache_delete()
        signals.satchmo_cart_add_verify.disconnect(vetoAllListener)

    def testCartAddVerifyVeto(self):
        """Test that vetoes from `signals.satchmo_cart_add_verify` are caught and cause an error."""
        try:
            site = Site.objects.get_current()
            cart = Cart(site=site)
            cart.save()
            p = Product.objects.get(slug='dj-rocks-s-b')
            cart.add_item(p, 1)
            order = make_test_order(self.US, '', include_non_taxed=True)
            self.fail('Should have thrown a CartAddProhibited error')

        except CartAddProhibited, cap:
            pass

        self.assertEqual(len(cart), 0)


if __name__ == "__main__":
    import doctest
    doctest.testmod()



#!/usr/bin/env python
# -*- coding: utf-8 -*-


from django.shortcuts import render_to_response
from notmm.dbapi.orm import decorators, sql

from notmm.utils.configparse import loadconf

from sandbox                import configuration as c

app_conf = loadconf('development.ini', section='moviereviews')
verbose = (app_conf['verbose'] == "true")

# generic view with positional arguments and kwargs
def index(request, **kwargs):
    return render_to_response(request, **kwargs)

# faked 500 error handling
def croak(request, template_name='500.mako'):
    return render_to_response(request, template_name, status=500)

# experimental scoped_session support for Django views
@with_session(engine=c.engine)
def test_session(request, **kwds):

    template_name = kwds['template_name']
    # app._scoped_session
    assert request.environ['_scoped_session'] != None

    return render_to_response(request, template_name)

@with_schevo_database(app_conf)
def test_schevo_database(request, **kwds):
    print 'in test_schevo_database'
    db = request.environ['schevo.db.moviereviews']
    assert db.format == 2, 'invalid db format: %d' % db.format
    assert hasattr(db, 'Q'), 'db has no Q attribute!'
    Q = db.Q
    movies = (item for item in db.Movie.find())
    #if verbose:
    #    for x in movies:
    #        print x
    db.close()
    return render_to_response(request, **kwds)
    



from satchmo_store.contact.supplier.models import RawItem, SupplierOrder, SupplierOrderItem, SupplierOrderStatus
from django.contrib import admin
from django.utils.translation import get_language, ugettext_lazy as _


class SupplierOrderItem_Inline(admin.TabularInline):
    model = SupplierOrderItem
    extra = 3

class SupplierOrderStatus_Inline(admin.StackedInline):
    model = SupplierOrderStatus
    extra = 1

class RawItemOptions(admin.ModelAdmin):
    list_display = ('supplier','description','supplier_num','inventory',)
    list_filter = ('supplier',)

class SupplierOrderOptions(admin.ModelAdmin):
    list_display = ('supplier','date_created', 'order_total','status')
    list_filter = ('date_created','supplier',)
    date_hierarchy = 'date_created'
    inlines = [SupplierOrderItem_Inline, SupplierOrderStatus_Inline]

admin.site.register(RawItem, RawItemOptions)
admin.site.register(SupplierOrder, SupplierOrderOptions)




from satchmo_ext.newsletter.models import Subscription, SubscriptionAttribute
from django.contrib import admin
from django.utils.translation import get_language, ugettext_lazy as _
    
class SubscriptionAttributeInline(admin.TabularInline):
    model = SubscriptionAttribute
    extra = 1

class SubscriptionOptions(admin.ModelAdmin):
    list_display = ['email', 'subscribed', 'create_date', 'update_date']
    list_filter = ['subscribed']
    inlines = [SubscriptionAttributeInline,]

admin.site.register(Subscription, SubscriptionOptions)




from django import template

from livesettings import config_get_group
from satchmo_utils.templatetags import get_filter_args

register = template.Library()

def order_details(context, order, default_view_tax=False):
    """Output a formatted block giving order details."""
    return {'order' : order,
        'default_view_tax' : default_view_tax,
        'request' : context['request']
    }

register.inclusion_tag('shop/_order_details.html', takes_context=True)(order_details)

def order_tracking_details(order, paylink=False, default_view_tax=False):
    """Output a formatted block giving order tracking details."""
    return {'order' : order,
        'default_view_tax': default_view_tax,
        'paylink' : paylink }

register.inclusion_tag('shop/_order_tracking_details.html')(order_tracking_details)

def order_variable(order, args):
    """
    Get a variable from an order

    Sample usage::

      {{ order|order_variable:'variable' }}

    """
    args, kwargs = get_filter_args(args)
    if not len(args == 1):
        raise template.TemplateSyntaxError("%r filter expected variable, got: %s" % (args[0], args))

    return order.get_variable(args[0])

register.filter(order_variable)

# def giftcertificate(order):
#     """Get the giftcertificate from the order, if any"""
#     try:
#         return GiftCertificate.objects.from_order(order)
#     except GiftCertificate.DoesNotExist:
#         pass
#
#     return None
#
# register.filter(giftcertificate)

@register.filter
def order_payment_methods(order):
    """
    Returns a list of payment method labels for an order.

    Usage::

      {{ order|order_payment_methods|join:", " }}

    """
    return (config_get_group('PAYMENT_%s' % p.payment).LABEL.value
             for p in order.payments.all())



import django.dispatch

shipping_data_query = django.dispatch.Signal()
# shipping_choices_query.send(sender=cart, cart=cart, 
#     paymentmodule=paymentmodule, contact=contact, 
#     default_view_tax=default_view_tax, order=order,
#     shipping_options = shipping_options,
#     shipping_dict = shipping_dict)
shipping_choices_query = django.dispatch.Signal()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2007-2013 Etienne Robillard <erob@gthcfoundation.org>
# All rights reserved.
# <LICENSE=ISC>
"""Customized wrappers over ``mako.lookup.TemplateLookup``.

Provides Mako templates loading and rendering extra customizations
to extend the ``TemplateLookup`` class.

"""

import mako.lookup as model


__all__ = ('UnicodeTemplateLoader', 'BaseTemplateLoader', 'CachedTemplateLoader')
class BaseTemplateLoader(model.TemplateLookup):
    """Minimal and fast template loader based on TemplateLookup."""
    collection_size = 48
    cache_enabled = False       # False=Disables template caching by default
    filesystem_checks = False   # False=Disables template reloading

class UnicodeTemplateLoader(BaseTemplateLoader):
    """Preconfigured template loader with generic Unicode bytestrings support"""
    default_filters = ['unicode'] # 'decode.utf8'
    input_encoding = 'UTF-8'
    encoding_errors = 'xmlcharrefreplace' # Rewrite Unicode encoding errors (when possible)
    #encoding_errors = 'replace'

class CachedTemplateLoader(UnicodeTemplateLoader):
    """Enables templates caching using selected cache backend (beaker)"""
    cache_enabled = True
    cache_impl='beaker'



from django.conf.urls.defaults import patterns as patterns_func
from django.utils.translation import ugettext
import product
from product import signals as product_signals
from product.modules.downloadable.models import DownloadLink
from satchmo_store.contact import signals as contact_signals
from satchmo_store.shop import notification
from satchmo_store.shop import signals
from satchmo_store.shop.listeners import recalc_total_on_contact_change, decrease_inventory_on_sale

from signals_ahoy.signals import collect_urls

import logging

log = logging.getLogger('product.modules.downloadable.listeners')

def add_download_urls(sender=None, patterns=None, section=None, **kwargs):
    # override top-level one, so that we don't get the PRODUCT_SLUG prefix for
    # backward-compatibility.
    if not section == '__init__':
        return

    urlpatterns = patterns_func('product.modules.downloadable.views',
        (r'^download/process/(?P<download_key>\w+)/$', 'process', {}, 'satchmo_download_process'),
        (r'^download/send/(?P<download_key>\w+)/$', 'send_file', {}, 'satchmo_download_send'),
    )

    if patterns:
        patterns += urlpatterns

def create_download_link(product=None, order=None, subtype=None, **kwargs):
    """Creates a download link for a Downloadable Product on order success."""
    if product and order and subtype == "download":
        new_link = DownloadLink(downloadable_product=product, order=order,
            key=product.create_key(), num_attempts=0)
        new_link.save()
    else:
        log.debug("ignoring subtype_order_success signal, looking for download product, got %s", subtype)


def ship_downloadable_order(order=None, **kwargs):
    if order.is_downloadable and not order.status == 'Shipped':
        order.add_status('Shipped', ugettext("Order immediately available for download"))

    product_signals.subtype_order_success.connect(create_download_link, sender=None)
    contact_signals.satchmo_contact_location_changed.connect(recalc_total_on_contact_change, sender=None)
    signals.order_success.connect(decrease_inventory_on_sale)
    signals.order_success.connect(notification.order_success_listener, sender=None)
    signals.order_success.connect(ship_downloadable_order, sender=None)

def start_default_listening():
    """Add required default listeners"""
    collect_urls.connect(add_download_urls, sender=product)
    product_signals.subtype_order_success.connect(create_download_link, sender=None)
    signals.order_success.connect(ship_downloadable_order, sender=None)

    log.debug('Added downnloadable product listeners')



from livesettings import ConfigurationGroup, config_register_list, IntegerValue, BooleanValue
from django.utils.translation import ugettext_lazy as _

THUMB_GROUP = ConfigurationGroup('THUMBNAIL', _('Thumbnail Settings'))

config_register_list(
    IntegerValue(THUMB_GROUP, 
        'IMAGE_QUALITY', 
        description= _("Thumbnail quality"), 
        help_text = _("Use a 1-100 value here, which will change the quality of JPG thumbnails created for products and categories."),
        default=75,
        ordering=0),
        
    BooleanValue(THUMB_GROUP,
        'RENAME_IMAGES',
        description=_("Rename product images?"),
        help_text=_("Automatically rename product images on upload?"),
        default=True)
)



'''
Default entity collection implementation
'''
import sys
import re

from elixir.py23compat import rsplit

class BaseCollection(list):
    def __init__(self, entities=None):
        list.__init__(self)
        if entities is not None:
            self.extend(entities)

    def extend(self, entities):
        for e in entities:
            self.append(e)

    def clear(self):
        del self[:]

    def resolve_absolute(self, key, full_path, entity=None, root=None):
        if root is None:
            root = entity._descriptor.resolve_root
        if root:
            full_path = '%s.%s' % (root, full_path)
        module_path, classname = rsplit(full_path, '.', 1)
        module = sys.modules[module_path]
        res = getattr(module, classname, None)
        if res is None:
            if entity is not None:
                raise Exception("Couldn't resolve target '%s' <%s> in '%s'!"
                                % (key, full_path, entity.__name__))
            else:
                raise Exception("Couldn't resolve target '%s' <%s>!"
                                % (key, full_path))
        return res

    def __getattr__(self, key):
        return self.resolve(key)

# default entity collection
class GlobalEntityCollection(BaseCollection):
    def __init__(self, entities=None):
        # _entities is a dict of entities keyed on their name.
        self._entities = {}
        super(GlobalEntityCollection, self).__init__(entities)

    def append(self, entity):
        '''
        Add an entity to the collection.
        '''
        super(EntityCollection, self).append(entity)

        existing_entities = self._entities.setdefault(entity.__name__, [])
        existing_entities.append(entity)

    def resolve(self, key, entity=None):
        '''
        Resolve a key to an Entity. The optional `entity` argument is the
        "source" entity when resolving relationship targets.
        '''
        # Do we have a fully qualified entity name?
        if '.' in key:
            return self.resolve_absolute(key, key, entity)
        else:
            # Otherwise we look in the entities of this collection
            res = self._entities.get(key, None)
            if res is None:
                if entity:
                    raise Exception("Couldn't resolve target '%s' in '%s'"
                                    % (key, entity.__name__))
                else:
                    raise Exception("This collection does not contain any "
                                    "entity corresponding to the key '%s'!"
                                    % key)
            elif len(res) > 1:
                raise Exception("'%s' resolves to several entities, you should"
                                " use the full path (including the full module"
                                " name) to that entity." % key)
            else:
                return res[0]

    def clear(self):
        self._entities = {}
        super(GlobalEntityCollection, self).clear()

# backward compatible name
EntityCollection = GlobalEntityCollection

_leading_dots = re.compile('^([.]*).*$')

class RelativeEntityCollection(BaseCollection):
    # the entity=None does not make any sense with a relative entity collection
    def resolve(self, key, entity):
        '''
        Resolve a key to an Entity. The optional `entity` argument is the
        "source" entity when resolving relationship targets.
        '''
        full_path = key

        if '.' not in key or key.startswith('.'):
            # relative target

            # any leading dot is stripped and with each dot removed,
            # the entity_module is stripped of one more chunk (starting with
            # the last one).
            num_dots = _leading_dots.match(full_path).end(1)
            full_path = full_path[num_dots:]
            chunks = entity.__module__.split('.')
            chunkstokeep = len(chunks) - num_dots
            if chunkstokeep < 0:
                raise Exception("Couldn't resolve relative target "
                    "'%s' relative to '%s'" % (key, entity.__module__))
            entity_module = '.'.join(chunks[:chunkstokeep])

            if entity_module and entity_module is not '__main__':
                full_path = '%s.%s' % (entity_module, full_path)

            root = ''
        else:
            root = None
        return self.resolve_absolute(key, full_path, entity, root=root)

    def __getattr__(self, key):
        raise NotImplementedError




"""Load up the registered plugins and plugin points.

New model instances will be created for registered poings and plugins.
Any unregistered plugins for a registered point will have an instance created.
Any unregistered points for a registered plugin will have an instance created.

Registered plugins and points which are no longer in the library system
are marked unregistered and un-enabled.


FUTURE:
    We could try to do a file system search for all unregistered plugins and
determine the connected points, but we would not be able to find db plugins
easily or those provided by more custom loaders. At some future point we may
consider having a loader extension API for listing unregistered plugins.

"""
from django.conf import settings
from django.template import loader, TemplateDoesNotExist
#from library import libraries
#from models import Plugin, PluginPoint, REMOVED, ENABLED
#from models import construct_template_path

from django.core.management.base import NoArgsCommand
from optparse import make_option

class Command(NoArgsCommand):
    option_list = NoArgsCommand.option_list + (
        make_option('--delete', action='store_true', dest='delete',
            help='delete the REMOVED Plugin and PluginPoint instances. '),
    )
    help = ("Syncs the registered plugin and plugin points with the model "
            "versions.")

    requires_model_validation = True

    def handle_noargs(self, **options):
        sync_app_plugins(options.get('delete', False))

def sync_app_plugins(delete_removed=False, verbosity=1):
    """
    We need to do a complicated dance to sync the current registered and
    unregistered implicit plugins and points with the instances in the DB.
    Plugins and points which are removed (or can no longer be found) are
    marked REMOVED (sometimes only to be re-enabed shortly there after).
    There is the option to delete all entries which are marked removed, but
    this should be done carefully. Other apps may have relations to the
    plugins and points for user preferences and customizaiton settings.
    We do not want to blindly delete these in a chain delete, as they may need
    to be migrated to a renamed plugin or point which has now been created as
    part of the sync.

    The process:
        1. for each registered Plugin Point
        1.1. if no instance exists, create it enabled.
        1.2. mark it registered
        1.2. if it was 'removed' then mark it enabled
        2. For each non-removed, registered instance for which there was no reg
        2.1. mark removed
        2.2. mark plugins removed
        3. for each registered plugin
        3.1. if there is no point, create it as unregistered
        3.2. if there is no plugin, create it as reg and enabled, w/template
        3.3. test the template load
        3.4. if the plugin was marked REMOVED
        3.4.1. mark it enabled
        3.4.2. if its point is REMOVED and it is not marked reg, enable it
        4. for each registered plugin for which there is no registration
        4.1. mark it removed
        5. for each non-REMOVED point, get all unregistered plugins via load:
        5.1. if plugin instance does not exist, create unregistered and enable
        5.2. if it does exist and is marked REMOVED, mark unreg-enabled w/ point
        6. for each unreg-non-removed point
        6.1. if all plugins are REMOVED, mark REMOVED.
        7. if asked to delete the removed, do so.
    """
    from app_plugins.library import libraries
    from app_plugins.models import Plugin, PluginPoint, REMOVED, ENABLED
    from app_plugins.models import construct_template_path

    instances = dict((p.label, p) for p in PluginPoint.objects.all())

    ## section 1 - registered plugin points
    for app_label, lib in libraries.iteritems():
        for label in lib.plugin_points:
            pp = instances.pop(label, None)
            if pp is None:
                if verbosity > 1:
                    print "Creating registered PluginPoint:", label
                pp = PluginPoint(label=label)
            pp.registered = True
            if pp.status == REMOVED:
                if verbosity > 1:
                    print "Updating registered PluginPoint:", label
                # re-enable a previously removed plugin point and its plugins
                pp.status = ENABLED
                for p in Plugin.objects.filter(point=pp, status=REMOVED):
                    p.status = ENABLED
                    p.save()
            pp.save()
            # search for unregistered plugins we do not yet know about?

    ## section 2 - removed plugin points
    for pp in instances.itervalues():
        if pp.status != REMOVED:
            pp.status = REMOVED
            pp.save()
            for p in pp.plugin_set.all():
                p.status = REMOVED
                p.save()

    instances = dict((p.label, p) for p in Plugin.objects.all())

    ## section 3 - registered plugins
    for app_label, lib in libraries.iteritems():
        for label in lib.plugins:
            p = instances.pop(label, None)
            point_label = label[len(lib.app_name):]
            if p is None:
                p = Plugin()
                p.label = label
                if verbosity > 1:
                    print "Creating registered Plugin:", label
                try:
                    point = PluginPoint.objects.get(label=point_label)
                    p.point = point
                    if point.status == REMOVED:
                        # point was removed at some point...
                        point.status = ENABLED
                        if point.registered:
                            point.registered = False
                        point.save()
                except PluginPoint.DoesNotExist:
                    if verbosity > 1:
                        print "Creating unregistered PluginPoint:", point_label
                    point = PluginPoint(label=point_label)
                    point.save()
                    p.point = point
            p.registered = True
            if p.status == REMOVED:
                # re-enable a previously removed plugin
                if verbosity > 1:
                    print "Updating registered Plugin:", p.label
                p.status = ENABLED
            options = lib.get_plugin_call(point_label).options
            default = construct_template_path(lib.app_name, point_label,
                                              options.get('ext', '.html'))
            # raise an error if it does not exist...
            template = options.get('template', default)
            loader.find_template_source(template)
            p.template = template
            p.save()

    ## section 4 - initial marking of unregistered known plugins
    for p in instances.itervalues():
        if p.status != REMOVED:
            p.status = REMOVED
            p.save()

    ## section 5 - unregistered plugins
    instances = dict((p.label, p) for p in Plugin.objects.all())
    for pp in PluginPoint.objects.exclude(status=REMOVED):
        ext = pp.get_options().get('ext', '.html')
        name = pp.label
        for app in settings.INSTALLED_APPS:
            label = u'.'.join([app, name])
            template = construct_template_path(app, name, ext)
            bFound = True
            try:
                loader.find_template_source(template)
            except TemplateDoesNotExist:
                bFound = False
            p = instances.get(label, None)
            if p is None:
                if bFound:
                    if verbosity > 1:
                        print "Creating unregistered Plugin:", label
                    p = Plugin(point=pp, label=label, template=template)
            else:
                if p.status == REMOVED and bFound:
                    p.status = ENABLED
                    p.template = template
                    p.registered = False
                    #if verbosity > 1:
                    #    print "Updating unregistered Plugin:", label
                elif not p.registered and not bFound and p.status != REMOVED:
                    p.status = REMOVED
                else:
                    p = None
            if p is not None:
                p.save()

    ## section 6 - removed unregistered plugin points
    for pp in PluginPoint.objects.filter(registered=False).exclude(status=REMOVED):
        if not pp.plugin_set.exclude(status=REMOVED).count():
            if verbosity > 1:
                print "Removing unregistered PluginPoint:", pp.label
            pp.status = REMOVED
            pp.save()

    ## section 7 - delete removed
    if delete_removed:
        count = Plugin.objects.filter(status=REMOVED).count()
        if count:
            if verbosity > 1:
                print "Deleting %d Removed Plugins" % count
            Plugin.objects.filter(status=REMOVED).delete()
        count = PluginPoints.objects.filter(status=REMOVED).count()
        if count:
            if verbosity > 1:
                print "Deleting %d Removed PluginPoint" % count
            PluginPoints.objects.filter(status=REMOVED).delete()



from django.utils.translation import ugettext_lazy as _
from livesettings import *

SHIP_MODULES = config_get('SHIPPING', 'MODULES')
SHIP_MODULES.add_choice(('shipping.modules.flat', _('Flat rate')))
SHIPPING_GROUP = config_get_group('SHIPPING')

config_register_list(

    DecimalValue(SHIPPING_GROUP,
        'FLAT_RATE',
        description=_("Flat shipping"),
        requires=SHIP_MODULES,
        requiresvalue='shipping.modules.flat',
        default="4.00"),

    StringValue(SHIPPING_GROUP,
        'FLAT_SERVICE',
        description=_("Flat Shipping Service"),
        help_text=_("Shipping service used with Flat rate shipping"),
        requires=SHIP_MODULES,
        requiresvalue='shipping.modules.flat',
        default=u"U.S. Mail"),
    
    StringValue(SHIPPING_GROUP,
        'FLAT_DAYS',
        description=_("Flat Delivery Days"),
        requires=SHIP_MODULES,
        requiresvalue='shipping.modules.flat',
        default="3 - 4 business days")
)



from inspect import getmembers, ismethod

from gazpacho.loader.loader import ObjectBuilder

class Proxy(object):
    def __init__(self, gladefile=None, gladestream=None, root=None):
        self._wt = ObjectBuilder(filename=gladefile, buffer=gladestream,
                                 root=root)

        self._signal_magicconnect()

    def signal_autoconnect(self, dic):
        self._wt.signal_autoconnect(dic)

    def _signal_magicconnect(self):
        """Look for our methods to see if we need to connect any signal.
        This is copied from Kiwi."""
        actions = []
        if self._uimanager is not None:
            for a in [ag.list_actions() for ag in \
                      self._uimanager.get_action_groups()]:
                actions.extend(a)
        self._connect_methods('on_', actions)
        self._connect_methods('after_', actions)

    def _connect_methods(self, method_prefix, actions):
        methods = [m for m in getmembers(self) \
                   if ismethod(m[1]) and m[0].find(method_prefix) == 0]

        for name, method in methods:
            index = name.rfind('__')
            widget_name = name[len(method_prefix):index]
            signal_name = name[index+2:]

            try:
                # First we look in the widgets
                widget = self._wt.get_widget(widget_name)
                if not widget:
                    raise AttributeError
                if method_prefix == 'on_':
                    widget.connect(signal_name, method)
                elif method_prefix == 'after_':
                    widget.connect_after(signal_name, method)
            except AttributeError:
                # Now we try to find it in the actions
                for action in actions:
                    if widget_name == action.get_name():
                        if method_prefix == 'on_':
                            action.connect(signal_name, method)
                        elif method_prefix == 'after_':
                            action.connect_after(signal_name, method)
                        break
                else:
                    print(('Warning: the widget %s is not on my widget tree '
                           'neither in the action list') % widget_name)

    def __getattr__(self, name):
        """Easy way to access the widgets by their name."""
        return self._wt.get_widget(name)

    def get_widgets(self):
        """Returns an iterator to loop through the widgets."""
        return self._wt.widgets

    widgets = property(get_widgets)



from datetime import datetime, timedelta
from decimal import Decimal
from django.core import urlresolvers
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _
from livesettings import config_get_group, config_value
from payment import active_gateways
from payment.forms import PaymentMethodForm, CustomChargeForm
from payment.views import contact
from satchmo_store.shop.models import Order, OrderItem, OrderPayment
from satchmo_utils.dynamic import lookup_url
from satchmo_utils.views import bad_or_missing
import logging

log = logging.getLogger('payment.views.balance')

def balance_remaining_order(request, order_id=None):
    """Load the order into the session, so we can charge the remaining amount"""
    # this will create an "OrderCart" - a fake cart to allow us to check out
    request.session['cart'] = 'order'
    request.session['orderID'] = order_id
    return balance_remaining(request)

def balance_remaining(request):
    """Allow the user to pay the remaining balance."""
    order = None
    orderid = request.session.get('orderID')
    if orderid:
        try:
            order = Order.objects.get(pk=orderid)
        except Order.DoesNotExist:
            # TODO: verify user against current user
            pass
            
    if not order:
        url = urlresolvers.reverse('satchmo_checkout-step1')
        return HttpResponseRedirect(url)

    if request.method == "POST":
        new_data = request.POST.copy()
        form = PaymentMethodForm(data=new_data, order=order)
        if form.is_valid():
            data = form.cleaned_data
            modulename = data['paymentmethod']
            if not modulename.startswith('PAYMENT_'):
                modulename = 'PAYMENT_' + modulename
            
            paymentmodule = config_get_group(modulename)
            url = lookup_url(paymentmodule, 'satchmo_checkout-step2')
            return HttpResponseRedirect(url)
        
    else:
        form = PaymentMethodForm(order=order)
        
    ctx = RequestContext(request, {'form' : form, 
        'order' : order,
        'paymentmethod_ct': len(active_gateways())
    })
    return render_to_response('shop/checkout/balance_remaining.html',
                              context_instance=ctx)


def charge_remaining(request, orderitem_id):
    """Given an orderitem_id, this returns a confirmation form."""
    
    try:
        orderitem = OrderItem.objects.get(pk = orderitem_id)
    except OrderItem.DoesNotExist:
        return bad_or_missing(request, _("The orderitem you have requested doesn't exist, or you don't have access to it."))
        
    amount = orderitem.product.customproduct.full_price
        
    data = {
        'orderitem' : orderitem_id,
        'amount' : amount,
        }
    form = CustomChargeForm(data)
    ctx = RequestContext(request, {'form' : form})
    return render_to_response('payment/admin/charge_remaining_confirm.html',
                              context_instance=ctx)

def charge_remaining_post(request):
    if not request.method == 'POST':
        return bad_or_missing(request, _("No form found in request."))
    
    data = request.POST.copy()
    
    form = CustomChargeForm(data)
    if form.is_valid():
        data = form.cleaned_data
        try:
            orderitem = OrderItem.objects.get(pk = data['orderitem'])
        except OrderItem.DoesNotExist:
            return bad_or_missing(request, _("The orderitem you have requested doesn't exist, or you don't have access to it."))
        
        price = data['amount']
        line_price = price*orderitem.quantity
        orderitem.unit_price = price
        orderitem.line_item_price = line_price
        orderitem.save()
        #print "Orderitem price now: %s" % orderitem.line_item_price
        
        order = orderitem.order
    
        if not order.shipping_cost:
            order.shipping_cost = Decimal("0.00")
    
        if data['shipping']:
            order.shipping_cost += data['shipping']
            
        order.recalculate_total()
        
        request.user.message_set.create(message='Charged for custom product and recalculated totals.')

        notes = data['notes']
        if not notes:
            notes = 'Updated total price'
            
        order.add_status(notes=notes)
        
        return HttpResponseRedirect('/admin/shop/order/%i' % order.id)
    else:
        ctx = RequestContext(request, {'form': form})
        return render_to_response('admin/charge_remaining_confirm.html',
                                  context_instance=ctx)



from notmm.utils.urlmap import RegexURLMap, include

#from product.urls.base import urlpatterns as productpatterns
from satchmo_store import shop
from satchmo_store.shop.views.sitemaps import sitemaps
from signals_ahoy.signals import collect_urls

#urlpatterns = shop.get_satchmo_setting('SHOP_URLS')

urlpatterns = RegexURLMap()
urlpatterns.add_routes('satchmo_store.shop.views',
    (r'^$','home.home', {}, 'satchmo_shop_home'),
    (r'^add/$', 'smart.smart_add', {}, 'satchmo_smart_add'),
    (r'^cart/$', 'cart.display', {}, 'satchmo_cart'),
    (r'^cart/accept/$', 'cart.agree_terms', {}, 'satchmo_cart_accept_terms'),
    (r'^cart/add/$', 'cart.add', {}, 'satchmo_cart_add'),
    (r'^cart/add/ajax/$', 'cart.add_ajax', {}, 'satchmo_cart_add_ajax'),
    (r'^cart/qty/$', 'cart.set_quantity', {}, 'satchmo_cart_set_qty'),
    (r'^cart/qty/ajax/$', 'cart.set_quantity_ajax', {}, 'satchmo_cart_set_qty_ajax'),
    (r'^cart/remove/$', 'cart.remove', {}, 'satchmo_cart_remove'),
    (r'^cart/remove/ajax/$', 'cart.remove_ajax', {}, 'satchmo_cart_remove_ajax'),
    (r'^checkout/', include('payment.urls')),
    (r'^contact/$', 'contact.form', {}, 'satchmo_contact'),
    (r'^history/$', 'orders.order_history', {}, 'satchmo_order_history'),
    (r'^quickorder/$', 'cart.add_multiple', {}, 'satchmo_quick_order'),
    (r'^tracking/(?P<order_id>\d+)/$', 'orders.order_tracking', {}, 'satchmo_order_tracking'),
    (r'^search/$', 'search.search_view', {}, 'satchmo_search'),
)

# here we add product patterns directly into the root url
#urlpatterns.include(productpatterns)

urlpatterns.add_routes('',
    (r'^contact/thankyou/$','django.views.generic.simple.direct_to_template',
        {'template':'shop/contact_thanks.html'},'satchmo_contact_thanks'),
    (r'^sitemap\.xml$', 'django.contrib.sitemaps.views.sitemap',
        {'sitemaps': sitemaps},
        'satchmo_sitemap_xml'),

)

# here we are sending a signal to add patterns to the base of the shop.
# collect_urls.send(sender=shop, patterns=urlpatterns)



from decimal import Decimal
from django import template
from l10n.utils import moneyfmt
from satchmo_utils.templatetags import get_filter_args
from tax.utils import get_tax_processor
from threaded_multihost.threadlocals import get_thread_variable, set_thread_variable, get_current_user
import logging

log = logging.getLogger('tax.templatetags')

register = template.Library()

def _get_taxprocessor(request=None):
    taxprocessor = get_thread_variable('taxer', None)
    if not taxprocessor:
        if request:            
            user = request.user
            if user.is_authenticated():
                user = user
            else:
                user = None
        else:
            user = get_current_user()

        taxprocessor = get_tax_processor(user=user)    
        set_thread_variable('taxer', taxprocessor)
        
    return taxprocessor

class CartitemLineTaxedTotalNode(template.Node):
    def __init__(self, cartitem, currency):
        self.cartitem = cartitem
        self.currency = currency

    def render(self, context):
        taxer = _get_taxprocessor(context['request'])
        try:
            item = template.resolve_variable(self.cartitem, context)
        except template.VariableDoesNotExist:
            raise template.TemplateSyntaxError("No such variable: %s", self.cartitem)
        
        total = item.line_total + taxer.by_price(item.product.taxClass, item.line_total)
        
        if self.currency:
            return moneyfmt(total)
        return total

def cartitem_line_taxed_total(parser, token):
    """Returns the line total for the cartitem, with tax added.  If currency evaluates true,
    then return the total formatted through moneyfmt.
    
    Example::
    
        {% cartitem_line_taxed_total cartitem [currency] %}
    """
    tokens = token.contents.split()
    if len(tokens) < 2:
        raise template.TemplateSyntaxError, "'%s' tag requires a cartitem argument" % tokens[0]

    return CartitemLineTaxedTotalNode(tokens[1], tokens[2])

class CartTaxedTotalNode(template.Node):
    def __init__(self, cart, currency):
        self.cart = cart
        self.currency = currency

    def render(self, context):
        taxer = _get_taxprocessor(context['request'])
        try:
            cart = template.resolve_variable(self.cart, context)
        except template.VariableDoesNotExist:
            raise template.TemplateSyntaxError("No such variable: %s", self.cart)

        total = Decimal('0.00')
        for item in cart:
            total += item.line_total + taxer.by_price(item.product.taxClass, item.line_total)
        if self.currency:
            return moneyfmt(total)
        
        return total

def cart_taxed_total(parser, token):
    """Returns the line total for the cartitem, with tax added.  If currency evaluates true,
    then return the total formatted through moneyfmt.
    
    Example:: 
    
        {% cart_taxed_total cartitem [currency] %}
    """
    tokens = token.contents.split()
    if len(tokens) < 2:
        raise template.TemplateSyntaxError, "'%s' tag requires a cart argument" % tokens[0]

    return CartTaxedTotalNode(tokens[1], tokens[2])

class TaxRateNode(template.Node):
    """Retrieve the tax rate for a category"""
    def __init__(self, taxclass, order, digits):
        self.taxclass = taxclass
        self.order = order
        self.digits = digits
        
    def render(self, context):
        taxer = _get_taxprocessor(context['request'])
        if self.order:
            try:
                order = template.resolve_variable(self.order, context)
                taxer.order = order
            except template.VariableDoesNotExist:
                pass            

        pcnt = taxer.get_percent(taxclass=self.taxclass)
        if self.digits == 0:
            q = Decimal('0')
        else:
            if self.digits == 1:
                s = "0.1"
            else:
                s = "0." + "0" * self.digits-1 + "1"
            q = Decimal(s)
        return pcnt.quantize(q)

def tax_rate(parser, token):
    """Returns the tax rate for an order, by tax category.
    
    Example:: 
    
        {% tax_rate taxclass [order] [digits] %}
    """
    tokens = token.contents.split()
    if len(tokens) < 2:
        raise template.TemplateSyntaxError, "'%s' tag requires a taxclass argument" % tokens[0]
        
    taxclass = tokens[1]
    if len(tokens) > 2:
        order = tokens[2]
    else:
        order = None
        
    if len(tokens) > 3:
        digits = int(tokens[3])
    else:
        digits = 0
        
    return TaxRateNode(taxclass, order, digits)
    
class TaxedPriceNode(template.Node):
    """Returns the taxed price for an amount.
    """
    def __init__(self, price, currency, taxclass):
        self.price = price
        self.taxclass = taxclass
        self.currency = currency

    def render(self, context):
        taxer = _get_taxprocessor(context['request'])
        try:
            price = template.resolve_variable(self.price, context)
        except template.VariableDoesNotExist:
            raise template.TemplateSyntaxError("No such variable: %s", self.price)

        total = price + taxer.by_price(self.taxclass, price)
        
        if self.currency:
            return moneyfmt(total)
                        
        return total
        
def taxed_price(parser, token):
    """Returns the taxed price for an amount.  If currency evaluates true,
    then return the total formatted through moneyfmt.
    
    Example:: 
        
        {% taxed_price amount [currency] [taxclass] %}
    """
    tokens = token.contents.split()
    if len(tokens) < 2:
        raise template.TemplateSyntaxError, "'%s' tag requires an amount argument" % tokens[0]
    
    price = tokens[1]
    if len(tokens) > 2:
        currency = tokens[2]
    else:
        currency = False
    
    if len(tokens) > 3:
        taxclass = tokens[3]
    else:
        taxclass = "Default"

    return TaxedPriceNode(price, currency, taxclass)

# Commenting out because I don't believe this is used anywhere now.
#def with_tax(product):
#    """Returns the product unit price with tax"""
#    return taxed
    

register.tag('cartitem_line_taxed_total', cartitem_line_taxed_total)
register.tag('cart_taxed_total', cart_taxed_total)
register.tag('taxed_price', taxed_price)
register.tag('tax_rate', tax_rate)



"""
Create a unique user id given a first and last name.
First, we try simple concatenation of first and last name.
If that doesn't work, we add random numbers to the name
"""

from django.contrib.auth.models import User
from django.utils.encoding import smart_unicode, force_unicode
from htmlentitydefs import name2codepoint
from satchmo_utils import random_string
import re
import unicodedata

_is_alnum_re = re.compile(r'\w+')
_ID_MIN_LENGTH = 5  # minimum reasonable length for username
_ID_MAX_LENGTH = 30 # as defined in django.auth.contrib.models.User.username field

def _id_generator(first_name, last_name, email):
    def _alnum(s, glue=''):
        return glue.join(filter(len, _is_alnum_re.findall(s))).lower()
    # The way to generate id is by trying:
    #  1. username part of email
    #  2. ascii-transliterated first+last name
    #  3. whole email with non-alphanumerics replaced by underscore
    #  4. random string
    # Every try must return at least _ID_MIN_LENGTH chars to succeed and is truncated
    # to _ID_MAX_LENGTH. All IDs are lowercased.
    id = _alnum(email.split('@')[0])
    if len(id) >= _ID_MIN_LENGTH:
        yield id[:_ID_MAX_LENGTH]
    id = _alnum(unicodedata.normalize('NFKD', first_name + last_name).encode('ascii', 'ignore'))
    if len(id) >= _ID_MIN_LENGTH:
        yield id[:_ID_MAX_LENGTH]
    id = _alnum(email, glue='_')
    if len(id) >= _ID_MIN_LENGTH:
        yield id[:_ID_MAX_LENGTH]
    while True:
        yield _alnum('%s_%s' % (id[:_ID_MIN_LENGTH], random_string(_ID_MIN_LENGTH, True)))[:_ID_MAX_LENGTH]

def generate_id(first_name='', last_name='', email=''):
    valid_id = False
    gen = _id_generator(first_name, last_name, email)
    test_name = gen.next()
    while valid_id is False:
        try:
            User.objects.get(username=test_name)
        except User.DoesNotExist:
            valid_id = True
        else:
            test_name = gen.next()
    return test_name

# From http://www.djangosnippets.org/snippets/369/
def slugify(s, entities=True, decimal=True, hexadecimal=True,
   instance=None, slug_field='slug', filter_dict=None):
    s = smart_unicode(s)

    #character entity reference
    if entities:
        s = re.sub('&(%s);' % '|'.join(name2codepoint), lambda m: unichr(name2codepoint[m.group(1)]), s)

    #decimal character reference
    if decimal:
        try:
            s = re.sub('&#(\d+);', lambda m: unichr(int(m.group(1))), s)
        except:
            pass

    #hexadecimal character reference
    if hexadecimal:
        try:
            s = re.sub('&#x([\da-fA-F]+);', lambda m: unichr(int(m.group(1), 16)), s)
        except:
            pass

    #translate
    s = unicodedata.normalize('NFKD', s).encode('ascii', 'ignore')

    #replace unwanted characters
    #Added _ because this is a valid slug option
    s = re.sub(r'[^-a-z0-9_]+', '-', s.lower())

    #remove redundant -
    s = re.sub('-{2,}', '-', s).strip('-')

    slug = s
    if instance:
        def get_query():
            query = instance.__class__.objects.filter(**{slug_field: slug})
            if filter_dict:
                query = query.filter(**filter_dict)
            if instance.pk:
                query = query.exclude(pk=instance.pk)
            return query
        counter = 1
        while get_query():
            slug = "%s-%s" % (s, counter)
            counter += 1
    return slug




"""
This is a stub and used as the default processor.
It doesn't do anything but it can be used to build out another
interface.

See the authorizenet module for the reference implementation
"""
from django.utils.translation import ugettext_lazy as _
from payment.modules.base import BasePaymentProcessor, ProcessorResult, NOTSET

class PaymentProcessor(BasePaymentProcessor):

    def __init__(self, settings):
        super(PaymentProcessor, self).__init__('dummy', settings)

    def authorize_payment(self, order=None, testing=False, amount=NOTSET):
        """
        Make an authorization for an order.  This payment will then be captured when the order
        is set marked 'shipped'.
        """
        if order == None:
            order = self.order
            
        if amount == NOTSET:
            amount = order.balance
        
        cc = order.credit_card
        if cc:
            ccn = cc.decryptedCC
            ccv = cc.ccv
            if ccn == '4222222222222':
                if ccv == '222':
                    self.log_extra('Bad CCV forced')
                    payment = self.record_failure(amount=amount, transaction_id='2', 
                        reason_code='2', details='CCV error forced')                
                    return ProcessorResult(self.key, False, _('Bad CCV - order declined'), payment)
                else:
                    self.log_extra('Setting a bad credit card number to force an error')
                    payment = self.record_failure(amount=amount, transaction_id='2', 
                        reason_code='2', details='Credit card number error forced')                
                    return ProcessorResult(self.key, False, _('Bad credit card number - order declined'), payment)

        orderauth = self.record_authorization(amount=amount, reason_code="0")
        return ProcessorResult(self.key, True, _('Success'), orderauth)

    def can_authorize(self):
        return True

    def capture_payment(self, testing=False, amount=NOTSET):
        """
        Process the transaction and return a ProcessorResult:

        Example:
        >>> from livesettings import config_get_group
        >>> settings = config_get_group('PAYMENT_DUMMY')
        >>> from payment.modules.dummy.processor import PaymentProcessor
        >>> processor = PaymentProcessor(settings)
        # If using a normal payment module, data should be an Order object.
        >>> data = {}
        >>> processor.prepare_data(data)
        >>> processor.process()
        ProcessorResult: DUMMY [Success] Success
        """
        
        orderpayment = self.record_payment(amount=amount, reason_code="0")
        return ProcessorResult(self.key, True, _('Success'), orderpayment)


    def capture_authorized_payment(self, authorization, amount=NOTSET):
        """
        Given a prior authorization, capture remaining amount not yet captured.
        """
        if amount == NOTSET:
            amount = authorization.remaining()

        orderpayment = self.record_payment(amount=amount, reason_code="0", 
            transaction_id="dummy", authorization=authorization)
        
        return ProcessorResult(self.key, True, _('Success'), orderpayment)
        
    def release_authorized_payment(self, order=None, auth=None, testing=False):
        """Release a previously authorized payment."""
        auth.complete = True
        auth.save()
        return ProcessorResult(self.key, True, _('Success'))



try:
    from decimal import Decimal
except:
    from django.utils._decimal import Decimal

from django.core.exceptions import ImproperlyConfigured
from django.db.models import Q
from livesettings import config_value
from satchmo_store.contact.models import Contact
from l10n.models import AdminArea, Country
from satchmo_utils import is_string_like
from product.models import TaxClass
from models import TaxBoundry, TaxRate, Taxable
import logging
import re
import datetime

log = logging.getLogger('tax.us_sst')

class Processor(object):

    method = "us_sst"

    def __init__(self, order=None, user=None):
        """
        Any preprocessing steps should go here
        For instance, copying the shipping and billing areas
        """
        self.order = order
        self.user = user

    def _get_location(self):
        area=country=postal_code=None

        if self.order:
            street1 = self.order.ship_street1
            street2 = self.order.ship_street2
            city = self.order.ship_city
            state = self.order.ship_state
            postal_code = self.order.ship_postal_code
            country = self.order.ship_country
            log.debug('Using order country: %s' % country)
            if country == 'US':
                area = state

        elif self.user and self.user.is_authenticated():
            try:
                contact = Contact.objects.get(user=self.user)
                try:
                    area = contact.shipping_address.state
                    postal_code = contact.shipping_address.postal_code
                except AttributeError:
                    pass
                try:
                    country = contact.shipping_address.country.iso2code
                    log.debug('Using contact country: %s' % country)
                except AttributeError:
                    pass

            except Contact.DoesNotExist:
                log.debug('No contact entry found.')
                pass

        if country:
            try:
                country = Country.objects.get(iso2_code__exact=country)
                log.debug('Found country by ISO code: %s' % country)
            except Country.DoesNotExist:
                log.info("Couldn't find Country from string: %s", country)
                country = None

        if not country:
            from satchmo_store.shop.models import Config
            country = Config.objects.get_current().sales_country

        if area:
            try:
                area = AdminArea.objects.get(name__iexact=area,
                                             country=country)
            except AdminArea.DoesNotExist:
                try:
                    area = AdminArea.objects.get(abbrev__iexact=area,
                                                 country=country)
                except AdminArea.DoesNotExist:
                    log.info("Couldn't find AdminArea from string: %s", area)
                    area = None

        # We don't actually try and match 'A' records.
        street_name = None
        street_number = None
        street_prefix = None
        street_suffix = None
        odd_even = None
        zip_5 = None
        zip_4 = None

        if country.iso2_code == 'US':
            if '-' in postal_code:
                zip_5, zip_4 = postal_code.split('-')
                zip_5 = int(zip_5)
                zip_4 = int(zip_4)
            else:
                zip_5 = int(postal_code)
                zip_4 = None


        log.debug('Area: %s' % area)
        log.debug('Country: %s' % country)
        log.debug('Zip: %s-%s' % (zip_5, zip_4))
        return {
            'area':area,
            'country':country,
            #'street1': street1,
            #'street_number': street_number,
            #'street_name': street_name,
            #'street_prefix': street_prefix,
            #'street_suffix': street_suffix,
            'odd_even': odd_even,
            #'street2': street2,
            #'city':city,
            #'state': state,
            'postal_code': postal_code,
            'zip_4': zip_4,
            'zip_5': zip_5,
        }

    def get_percent(self, taxclass="Default", area=None, country=None):
        return 100*self.get_rate(taxclass=taxclass, area=area, country=country)

    def get_boundry(self, taxable_class, location, date=None):
        """
        Location is a dict of:
            'area', 'country': objects
            'street1', 'street2', 'city', 'state', 'postal_code': strings
        """
        boundry = TaxBoundry.lookup(location['zip_5'], location['zip_4'], date)
        log.debug('Boundry: %s' % boundry)
        return boundry

    def get_rate(self, taxclass=None, area=None, country=None, get_object=False, **kwargs):
        if not taxclass:
            taxclass = "Default"
        rate = None

        #if not (area or country):
        location = self._get_location()
        state = location['area']
        country = location['country']

        if is_string_like(taxclass):
            try:
                taxclass = TaxClass.objects.get(title__iexact=taxclass)

            except TaxClass.DoesNotExist:
                raise ImproperlyConfigured("Can't find a '%s' Tax Class", taxclass)

        # This module only works in the USA.
        if country.iso2_code != 'US':
            rate = None
        else:
            try:
                taxable = Taxable.objects.get(taxClass=taxclass, taxZone=state)
                if taxable.isTaxable is False:
                    rate = None
                else:
                    rate = self.get_boundry(taxable, location)
                    rate.useIntrastate = taxable.useIntrastate
                    rate.useFood = taxable.useFood

            except Taxable.DoesNotExist:
                rate = None

        log.debug("Got rate [%s] = %s", taxclass, rate)
        if get_object:
            return rate
        else:
            if rate:
                return rate.percentage
            else:
                return Decimal("0.00")

    def by_price(self, taxclass, price):
        rate = self.get_rate(taxclass)

        if not rate:
            t = Decimal("0.00")
        else:
            t = rate * price

        return t

    def by_product(self, product, quantity=Decimal('1')):
        """Get the tax for a given product"""
        price = product.get_qty_price(quantity)
        tc = product.taxClass
        return self.by_price(tc, price)

    def by_orderitem(self, orderitem):
        if orderitem.product.taxable:
            price = orderitem.sub_total
            taxclass = orderitem.product.taxClass
            return self.by_price(taxclass, price)
        else:
            return Decimal("0.00")

    #def shipping(self, subtotal=None):
    #    if subtotal is None and self.order:
    #        subtotal = self.order.shipping_sub_total
    #
    #    if subtotal:
    #        rate = None
    #        if config_value('TAX','TAX_SHIPPING'):
    #            try:
    #                tc = TaxClass.objects.get(title=config_value('TAX', 'TAX_CLASS'))
    #                rate = self.get_rate(taxclass=tc)
    #            except:
    #                log.error("'Shipping' TaxClass doesn't exist.")
    #
    #        if rate:
    #            t = rate * subtotal
    #        else:
    #            t = Decimal("0.00")
    #
    #    else:
    #        t = Decimal("0.00")
    #
    #    return t

    def process(self, order=None):
        """
        Calculate the tax and return it.

        Probably need to make a breakout.
        """
        if order:
            self.order = order
        else:
            order = self.order

        sub_total = Decimal('0.00')
        taxes = {}
        rates = {}

        items = [{'type':'item', 'item':x} for x in order.orderitem_set.filter(product__taxable=True)]
        items.append({'type':'shipping', 'price':self.order.shipping_sub_total})

        for i in items:
            if i['type'] == 'item':
                item = i['item']
                tc = item.product.taxClass
                price = item.sub_total
            else:
                if not config_value('TAX','TAX_SHIPPING'):
                    continue

                try:
                    tc = TaxClass.objects.get(title=config_value('TAX', 'TAX_CLASS'))
                    price = i['price']
                    #rate = self.get_rate(taxclass=tc)
                except:
                    log.error("'Shipping' TaxClass doesn't exist.")
                    continue

            if tc:
                tc_key = tc.title
            else:
                tc_key = "Default"

            if tc_key in rates:
                rate = rates[tc_key]
            else:
                rate = self.get_rate(tc, get_object=True)
                rates[tc_key] = rate

            for tax_rate in rate.rates():
                if rate.serCode:
                    tax_key = "%s (%s-SerCode-%s)" % (
                        tc_key, tax_rate.state, rate.serCode
                    )
                else:
                    tax_key = "%s (%s-%s-%s)" % (
                        tc_key, tax_rate.state, tax_rate.jurisdictionType,
                        tax_rate.jurisdictionFipsCode
                    )

                if not tax_key in taxes:
                    taxes[tax_key] = Decimal("0.00")


                if rate:
                    t = price*tax_rate.rate(intrastate=rate.useIntrastate, food=rate.useFood)
                else:
                    t = Decimal("0.00")
                sub_total += t
                taxes[tax_key] += t

        #ship = self.shipping()
        #sub_total += ship
        #taxes['Shipping'] = ship
        ## TODO: Fix this to do the same for shipping as items.

        for k in taxes:
            log.debug('Taxes: %s = %s' % (k, taxes[k]))
            taxes[k] = taxes[k]

        return sub_total, taxes
